import sys
import os

os.environ['DJANGO_SETTINGS_MODULE'] = 'test_settings'

import django
from django.conf import settings


django.setup()

from django.test.utils import get_runner

TestRunner = get_runner(settings)
test_runner = TestRunner(verbosity=2, failfast=False)
failures = test_runner.run_tests(['console'])

if failures:
    sys.exit(failures)



from __future__ import unicode_literals, absolute_import

from django.http import HttpResponseRedirect


def login_required(function):
    def wrap(request, *args, **kwargs):
        if (request.session.get('username', False) and
           request.session.get('password', False) and
           request.session.get('api_url', False)):
            return function(request, *args, **kwargs)
        else:
            return HttpResponseRedirect('/')

    wrap.__doc__ = function.__doc__
    wrap.__name__ = function.__name__

    return wrap



from __future__ import unicode_literals
from pprint import pformat
from datetime import date, timedelta

from django.test import TestCase
from django.conf import settings
from django import forms

from ..forms import LoginForm, MeForm, SSHForm, ContainerForm, DomainForm, CalendarForm
from ..models import UwsgiItApi


class FormTesterMixin():
    def assertFormError(self, form_cls, expected_error_name, expected_error_msg, data):
        test_form = form_cls(data=data)
        #if we get an error then the form should not be valid
        self.assertFalse(test_form.is_valid())
        self.assertEquals(
            test_form.errors[expected_error_name],
            expected_error_msg,
            msg="Expected {} : Actual {} : using data {}"
                .format(test_form.errors[expected_error_name],
                        expected_error_msg, pformat(data)))


class LoginFormTests(FormTesterMixin, TestCase):

    @classmethod
    def setUpClass(cls):
        cls.test_api = UwsgiItApi(
            url=settings.DEFAULT_API_URL,
            name='TEST API')
        cls.test_api.save()

    @classmethod
    def tearDownClass(cls):
        cls.test_api.delete()

    def test_LoginForm_data_validation_for_invalid_data(self):
        invalid_data_list = [
            {'data': {'username': 'test', 'api_url': self.test_api.id, 'action_login': 1},
                'error': ('password', ['This field is required.'])},
            {'data': {'password': '1234', 'api_url': self.test_api.id, 'action_login': 1},
                'error': ('username', ['This field is required.'])},
            {'data': {'username': 'test', 'password': '1234', 'action_login': 1},
                'error': ('api_url', ['This field is required.'])},
            {'data': {'username': 'test', 'api_url': self.test_api.id},
                'error': ('action_login', ['This field is required.'])}
        ]

        for invalid_data in invalid_data_list:
            self.assertFormError(
                LoginForm,
                invalid_data['error'][0],
                invalid_data['error'][1],
                invalid_data['data'])

    def test_LoginForm_data_validation_for_valid_data(self):
        form = LoginForm({'username': settings.TEST_USER, 'password': settings.TEST_PASSWORD,
                         'api_url': self.test_api.id, 'action_login': 1})
        self.assertTrue(form.is_valid())
        #this will throw an error if it doesn't clean correctly
        self.assertIsNotNone(form.clean())

    def test_LoginForm_with_wrong_credentials_throws_error(self):
        form = LoginForm({'username': 'test', 'password': '1234', 'action_login': 1, 'api_url': self.test_api.id})
        self.assertFalse(form.is_valid())
        self.assertRaisesMessage(forms.ValidationError, 'Wrong username or password', form.clean)


class MeFormTests(FormTesterMixin, TestCase):

    def test_MeForm_data_validation_for_invalid_data(self):
        invalid_data_list = [
            {'data': {'company': 'test', 're_password': '1234'},
                'error': ('password', ['This field is required.'])},
            {'data': {'password': '1234', 're_password': '1234', 'vat': 1},
                'error': ('company', ['This field is required.'])},
            {'data': {'company': 'test', 'password': '1234', 'vat': 1},
                'error': ('re_password', ['This field is required.'])},
        ]

        for invalid_data in invalid_data_list:
            self.assertFormError(
                MeForm,
                invalid_data['error'][0],
                invalid_data['error'][1],
                invalid_data['data'])

    def test_MeForm_data_validation_for_valid_data(self):
        form = MeForm({'company': 'test', 'password': '1234',
                         're_password': '1234', 'vat': 1})
        self.assertTrue(form.is_valid())
        #this will throw an error if it doesn't clean correctly
        self.assertIsNotNone(form.clean())

    def test_MeForm_with_different_passwords_throws_error(self):
        invalid_data = {
            'data': {'company': 'test', 're_password': '1234', 'password': '123'},
            'error': ('re_password', ['Passwords do not match'])}

        self.assertFormError(
            MeForm,
            invalid_data['error'][0],
            invalid_data['error'][1],
            invalid_data['data'])


class SSHFormTests(FormTesterMixin, TestCase):

    def test_SSHForm_data_validation_for_invalid_data(self):
        invalid_data_list = [
            {'data': {}, 'error': ('key', ['This field is required.'])}
        ]

        for invalid_data in invalid_data_list:
            self.assertFormError(
                SSHForm,
                invalid_data['error'][0],
                invalid_data['error'][1],
                invalid_data['data'])

    def test_SSHForm_data_validation_for_valid_data(self):
        keys = [
            {'key': 'ssh-rsa ' + 'ZZZB3NzaC1yc2EZZZDAQABAFFBAQCui4hJItITzlRHo'*3 + 'Test@Test.local'},
            {'key': '\n\tssh-rsa ' + 'ZZZB3NzaC1yc2EZZZDAQABAFFBAQCui4hJItITzlRHo'*3 + 'Test@Test.local'},
            {'key': 'ssh-dsa ' + 'ZZZB3NzaC1yc2EZZZDAQABAFFBAQCui4hJItITzlRHo'*3 + 'Test@Test.local      \n'},

        ]
        for key in keys:
            form = SSHForm(key)
            self.assertTrue(form.is_valid())
            #this will throw an error if it doesn't clean correctly
            self.assertIsNotNone(form.clean())

    def test_SSHForm_with_wrong_ssh_throws_error(self):
        keys = [
            {'data': {'key': 'ssh-rsa' + 'ZZZB3NzaC1yc2EZZZDAQABAFFBAQCui4hJItITzlRHo'*3 + ' Test@Test.local'},
                'error': 'Inserted value is not a valid ssh key'},
            {'data': {'key': 'ZZZB3NzaC1yc2EZZZDAQABAFFBAQCui4hJItITzlRHo'*3 + ' Test@Test.local'},
                'error': 'Inserted value is not a valid ssh key'},
            {'data': {'key': 'ssh-rs ' + 'ZZZB3NzaC1yc2EZZZDAQABAFFBAQCui4hJItITzlRHo'*3},
                'error': 'Inserted value is not a valid ssh key'},
            {'data': {'key': 'ssh-rsa ' + 'ZZZB3NzaC1yc2E\nZZZDAQABAFFBAQCui4hJItITzlRHo'*3 },
                'error': 'Too many newlines in the ssh key'},
            {'data': {'key': 'ssh-rsa a Test@Test.local'},
                'error': 'Key too short'}
        ]
        for key in keys:
            form = SSHForm(key['data'])
            self.assertFalse(form.is_valid())
            self.assertRaisesMessage(forms.ValidationError, key['error'], form.clean)


class ContainerFormTests(TestCase):

    def test_ContainerForm_data_validation_for_valid_data(self):
        form = ContainerForm({'tags': ['1'], 'linked_to': ['1'], 'distro': ['1'], 'quota_threshold': 90}, distro_choices=((1, 1), (2, 2)), tag_choices=((1, 1), (2, 2)), linked_to_choices=((1, 1), (2, 2)))
        self.assertTrue(form.is_valid())
        #this will throw an error if it doesn't clean correctly
        self.assertIsNotNone(form.clean())


class DomainFormTests(TestCase):

    def test_DomainForm_data_validation_for_valid_data(self):
        form = DomainForm({'did': 267, 'tags': [1]}, tag_choices=((1, 1), (2, 2)))
        self.assertTrue(form.is_valid())
        #this will throw an error if it doesn't clean correctly
        self.assertIsNotNone(form.clean())


class CalendarFormTests(FormTesterMixin, TestCase):

    @classmethod
    def setUpClass(cls):
        cls.today = date.today()

    def test_CalendarForm_data_validation_for_invalid_data(self):
        invalid_data_list = [
            {'data': {'year': 2014, 'day': 12},
                'error': ('month', ['Month is required.'])},
            {'data': {'month': 1},
                'error': ('year', ['This field is required.'])},
            {'data': {'month': 1, 'day': 12},
                'error': ('year', ['This field is required.'])},
            {'data': {'day': 12},
                'error': ('year', ['This field is required.'])},
        ]

        for invalid_data in invalid_data_list:
            self.assertFormError(
                CalendarForm,
                invalid_data['error'][0],
                invalid_data['error'][1],
                invalid_data['data'])

    def test_CalendarForm_data_validation_for_valid_data(self):
        valid_data_list = [
            {'year': self.today.year},
            {'year': self.today.year, 'month': self.today.month},
            {'year': self.today.year, 'month': self.today.month, 'day': self.today.day},
        ]
        for valid_data in valid_data_list:
            form = CalendarForm(valid_data)
            self.assertTrue(form.is_valid())
            #this will throw an error if it doesn't clean correctly
            self.assertIsNotNone(form.clean())

    def test_CalendarForm_time_unit_returns_right_value(self):
        valid_data_list = [
            {'data': {'year': self.today.year}, 'time_unit': 'month'},
            {'data': {'year': self.today.year, 'month': self.today.month}, 'time_unit': 'day'},
            {'data': {'year': self.today.year, 'month': self.today.month, 'day': self.today.day}, 'time_unit': 'hour'},
        ]
        for valid_data in valid_data_list:
            form = CalendarForm(valid_data['data'])
            self.assertEqual(form.is_valid(), True)
            self.assertEqual(form.time_unit(), valid_data['time_unit'])

    def test_CalendarForm_is_in_future_fails_with_date_in_the_future(self):
        one_year_in_future = self.today + timedelta(365)
        tomorrow = self.today + timedelta(365)
        invalid_data_list = [
            {'year': one_year_in_future.year},
            {'year': one_year_in_future.year, 'month': one_year_in_future.month},
            {'year': one_year_in_future.year, 'month': one_year_in_future.month, 'day': one_year_in_future.day},
            {'year': tomorrow.year, 'month': tomorrow.month, 'day': tomorrow.day},
        ]

        for invalid_data in invalid_data_list:
            form = CalendarForm(invalid_data)
            self.assertFalse(form.is_valid())
            self.assertRaisesMessage(forms.ValidationError, 'Set a date in the past.', form.clean)




'''
Django settings for uwsgiit_console example project.

For more information on this file, see
https://docs.djangoproject.com/en/1.6/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.6/ref/settings/
'''
import os


gettext = lambda s: s

BASE_DIR = os.path.dirname(os.path.dirname(__file__))

# Console configuration

CONSOLE_TITLE = 'uWSGI.it Console'
CONSOLE_SUBTITLE = ''
DEFAULT_API_URL = 'https://api.uwsgi.it/api/'

TEST_USER = '20tab'
TEST_PASSWORD = '20t4b@rgrL$&!Lgudm'
TEST_CONTAINER = '30005'
TEST_DOMAIN = '314'
TEST_TAG = '20tab'

######################

DEBUG = TEMPLATE_DEBUG = True

DJANGO_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
)

THIRD_PARTY_APPS = (
)

LOCAL_APPS = (
    'console',
)

INSTALLED_APPS = DJANGO_APPS + THIRD_PARTY_APPS + LOCAL_APPS

TEMPLATE_DIRS = (
    os.path.join(os.path.abspath(BASE_DIR), 'templates'),
)

MANAGERS = (('errors', 'errors@email.com'),)

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}


# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'j!7z%+(kog8tqv-%y7ga5a#0i+!mc_436p2u&i_v9uy1@!#^&t'

MIDDLEWARE_CLASSES = [
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

TEMPLATE_CONTEXT_PROCESSORS = (
    'django.contrib.auth.context_processors.auth',
    'django.core.context_processors.debug',
    'django.core.context_processors.i18n',
    'django.core.context_processors.media',
    'django.core.context_processors.static',
    'django.core.context_processors.tz',
    'django.contrib.messages.context_processors.messages',
    'console.template_context.context_processors.console_context',
)


ROOT_URLCONF = 'console.urls'

WSGI_APPLICATION = 'uwsgiit_console.wsgi.demo.application'


# Internationalization
# https://docs.djangoproject.com/en/1.6/topics/i18n/

LANGUAGE_CODE = 'en-us'

MODELTRANSLATION_DEFAULT_LANGUAGE = 'en'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.6/howto/static-files/

STATIC_ROOT = os.path.abspath(os.path.join(BASE_DIR, 'static'))
MEDIA_ROOT = os.path.abspath(os.path.join(BASE_DIR, 'media'))
STATIC_URL = '/static/'
MEDIA_URL = '/media/'



from __future__ import unicode_literals, absolute_import
from datetime import datetime
import json

from django.conf import settings
from django.contrib import messages
from django.template import RequestContext
from django.shortcuts import render_to_response
from django.http import HttpResponseRedirect, HttpResponse,\
    HttpResponseForbidden

from .utils import ConsoleClient as CC

from .decorators import login_required
from .forms import LoginForm, MeForm, SSHForm, ContainerForm, TagForm,\
    DomainForm, NewDomainForm, CalendarForm, LoopboxForm, NewLoopboxForm,\
    AlarmForm


def logout(request):
    request.session.flush()
    return HttpResponseRedirect('/')


def main_render(request, template, v_dict={}):
    username = request.session.get('username', False)
    password = request.session.get('password', False)
    api_url = request.session.get('api_url', False)

    if username and password and api_url:
        client = CC(username, password, api_url)

        v_dict['containers'] = sorted(
            client.containers().json(), key=lambda k: k['name'])
        last_alarm = client.alarms({'range': 1}).json()
        if last_alarm:
            v_dict['last_alarm_id'] = last_alarm[0]['id']

    return render_to_response(
        template, v_dict, context_instance=RequestContext(request))


def home(request):
    v_dict = {}
    if 'action_login' in request.POST:
        login_form = LoginForm(request.POST)
        if login_form.is_valid():
            cd = login_form.cleaned_data
            request.session['username'] = cd['username']
            request.session['password'] = cd['password']
            request.session['api_url'] = cd['api_url'].url
            return HttpResponseRedirect('/me/')
    else:
        login_form = LoginForm()

    username = request.session.get('username', None)
    password = request.session.get('password', None)
    api_url = request.session.get('api_url', settings.DEFAULT_API_URL)

    client = CC(username, password, api_url)

    news = client.news().json()
    for n in news:
        n['date'] = datetime.fromtimestamp(n['date'])

    v_dict['news'] = news

    if client.username is None:
        v_dict['login_form'] = login_form
    else:
        if 'del-alarm' in request.GET:
            client.delete_alarm(request.GET['del-alarm'])

        v_dict['alarms'] = client.alarms({'range': 10}).json()
        if 'error' in v_dict['alarms']:
            return logout(request)

        for a in v_dict['alarms']:
            a['unix'] = datetime.fromtimestamp(a['unix'])

    return main_render(request, 'console/index.html', v_dict)


@login_required
def me_page(request):
    client = CC(request.session.get('username'),
                request.session.get('password'),
                request.session.get('api_url'))

    me = client.me().json()
    if 'error' in me:
        return logout(request)

    v_dict = {'me': me}
    me_form = MeForm(initial={
        'company': me['company'],
        'email': me['email'],
        'password': request.session.get('password'),
        're_password': request.session.get('password'),
        'vat': me['vat']
    })

    if request.POST:
        me_form = MeForm(request.POST)
        if me_form.is_valid():
            cd = me_form.cleaned_data
            client.update_me({
                'company': cd['company'],
                'email': cd['email'],
                'password': cd['password'],
                'vat': cd['vat']
            })
            request.session['password'] = cd['password']

    v_dict['me_form'] = me_form
    v_dict['distros'] = client.distros().json()

    return main_render(request, 'console/me.html', v_dict)


@login_required
def containers(request, id):
    res = {}
    client = CC(request.session.get('username'),
                request.session.get('password'),
                request.session.get('api_url'))
    if id:
        container = client.container(id).json()
        if 'error' in container:
            return logout(request)

        # Get last quota metric
        quota_metrics = client.container_metric(id, 'quota', None).json()
        if len(quota_metrics) > 0:
            used_quota = quota_metrics[-1][1]
            used_quota /= 1024 * 1024
        # If there are no metrics (in case of new container)
        else:
            used_quota = 0

        res['container'] = container

        distros_list = sorted(client.distros().json(), key=lambda distro: distro['id'], reverse=True)
        distros_list = [(x['id'], x['name']) for x in distros_list]

        tag_list = client.get_tag_list()

        containers_actual_linked_to = [x for x in client.containers().json() if x['uid'] != int(id)]

        linked_to = [(x['uid'], '{} ({})'.format(
            x['name'], x['uid'])) for x in containers_actual_linked_to]

        containerform = ContainerForm(
            tag_choices=tag_list,
            linked_to_choices=linked_to,
            distro_choices=distros_list,
            initial=container
        )
        sshform = SSHForm()
        calendar = CalendarForm()
        newloopboxform = NewLoopboxForm()

        container_copy = {k: container[k] for k in container if k not in containerform.fields}
        del container_copy['ssh_keys']
        del container_copy['distro_name']

        container_copy['storage'] = str(used_quota) + ' / ' + str(container_copy['storage']) + ' MB'
        container_copy['memory'] = str(container_copy['memory']) + 'MB'

        res['container_copy'] = container_copy

        active_panel = None
        if request.POST:
            if 'distro' in request.POST:
                containerform = ContainerForm(
                    request.POST,
                    tag_choices=tag_list,
                    linked_to_choices=linked_to,
                    distro_choices=distros_list)

                if containerform.is_valid():
                    cd = containerform.cleaned_data
                    container_updates = {
                        'distro': cd['distro'],
                        'tags': cd['tags'],
                        'note': cd['note'],
                        'nofollow': cd['nofollow'],
                        'quota_threshold': cd['quota_threshold']
                    }

                    optional_values = (
                        'name', 'jid', 'jid_destinations', 'jid_secret',
                        'alarm_freq', 'pushover_user', 'pushover_token',
                        'pushover_sound'
                    )
                    for ov in optional_values:
                        if ov in cd:
                            container_updates[ov] = cd[ov]
                    if 'reboot' in cd and cd['reboot']:
                        container_updates['reboot'] = True
                    # client.update_container(id, container_updates)

                    list_linked_to = [x['uid'] for x in containers_actual_linked_to]

                    for link in cd['linked_to']:
                        if link not in list_linked_to:
                            client.update_container(id, {'link': link})
                    for link in list_linked_to:
                        if unicode(link) not in cd['linked_to']:
                            client.update_container(id, {'unlink': link})

            elif 'action' in request.POST:
                action = request.POST.get('action')
                active_panel = 'ssh'
                sshform = SSHForm(request.POST)
                if sshform.is_valid():
                    cd = sshform.cleaned_data
                    if action == 'add-key':
                        if cd['key'] not in container['ssh_keys']:
                            container['ssh_keys'].append(cd['key'].strip())
                            response = client.container_set_keys(id, container['ssh_keys'])
                            if response.status_code == 200:
                                messages.success(request, 'New key successfully added')
                            else:
                                messages.error(request, 'An error occurred, please try again')
                            sshform = SSHForm()
                        else:
                            msg = 'Key {key} was already added to container {id}'.format(key=cd['key'], id=id)
                            messages.warning(request, msg)
                    elif action == 'del-key':
                        cd['key'] = cd['key'].strip()
                        if cd['key'] in container['ssh_keys']:
                            container['ssh_keys'].remove(cd['key'])
                            response = client.container_set_keys(id, container['ssh_keys'])
                            if response.status_code == 200:
                                messages.success(request, 'Key successfully removed')
                            else:
                                messages.error(request, 'An error occurred, please try again')

            else:
                active_panel = 'loopboxes'
                if 'mountpoint' in request.POST:
                    newloopboxform = NewLoopboxForm(request.POST)
                    if newloopboxform.is_valid():
                        cd = newloopboxform.cleaned_data
                        r = client.create_loopbox(id, cd['filename'], cd['mountpoint'])
                        if r.uerror:
                            newloopboxform.add_error(None, 'An error occurred: {}'.format(r.json()['error']))
                elif 'lid' in request.POST:
                    loopbox_form = LoopboxForm(request.POST, tag_choices=tag_list)
                    if loopbox_form.is_valid():
                        cd = loopbox_form.cleaned_data
                        lid = cd['lid']
                        tags_post = request.POST.getlist('{}-tags'.format(lid))
                        client.update_loopbox(lid, {'tags': tags_post})

        elif request.GET:
            if 'del-loopbox' in request.GET:
                active_panel = 'loopboxes'
                client.delete_loopbox(request.GET['del-loopbox'])
            if 'del-alarm' in request.GET:
                active_panel = 'alarms'
                client.delete_alarm(request.GET['del-alarm'])

        res['alarms'] = client.alarms({'container': id}).json()

        alarms_used_keys = ('a_color', 'a_container', 'a_class', 'a_level', 'a_vassal', 'a_filename', 'a_func', 'a_line')
        for k in alarms_used_keys:
            res[k] = set()

        for a in res['alarms']:
            a['unix'] = datetime.fromtimestamp(a['unix'])
            for k in a:
                if k != 'unix' and k != 'id' and k != 'msg' and a[k]:
                    res['a_{}'.format(k)].add(a[k])

        for k in alarms_used_keys:
            if res[k]:
                if k == 'a_level':
                    levels = {0: 'System', 1: 'User', 2: 'Exception', 3: 'Traceback', 4: 'Log'}
                    res[k] = [{'id': x, 'text': levels[x]} for x in res[k] if res[k]]
                else:
                    res[k] = [{'id': x, 'text': str(x)} for x in res[k] if res[k]]
                res[k] = json.dumps([{'id': '', 'text': ''}] + res[k])
            else:
                del res[k]

        loopboxes = client.loopboxes(container=id).json()

        loopboxes_list = []
        used_tags = []

        for l in loopboxes:
            form = LoopboxForm(initial={'lid': l['id'], 'tags': l['tags']}, prefix=l['id'], tag_choices=tag_list)
            loopboxes_list.append((l, form))
            used_tags.extend([tag for tag in l['tags'] if tag not in used_tags])

        res['loopboxes'] = loopboxes_list
        res['tags'] = used_tags

        containerform.fields['tags'].initial = container['tags']
        containerform.fields['linked_to'].initial = container['linked_to']

        res['containerform'] = containerform
        res['sshform'] = sshform
        res['calendar'] = calendar
        res['newloopboxform'] = newloopboxform
        res['active_panel'] = active_panel
    return main_render(request, 'console/containers.html', res)


@login_required
def domains(request):
    res = {}
    client = CC(request.session.get('username'),
                request.session.get('password'),
                request.session.get('api_url'))

    new_domain = NewDomainForm()
    calendar = CalendarForm()

    all_tags = client.list_tags().json()
    if 'error' in all_tags:
        return logout(request)

    tags_list = client.get_tag_list()

    if request.POST:
        if 'name' in request.POST:
            new_domain = NewDomainForm(request.POST)
            if new_domain.is_valid():
                name = new_domain.cleaned_data['name']
                r = client.add_domain(name)
                if r.uerror:
                    new_domain.add_error(None, 'Error: {}'.format(r.json()['error']))
                else:
                    messages.success(
                        request, 'domain {} successfully added'.format(name))
                    new_domain = NewDomainForm()
    if 'del' in request.GET:
        name = request.GET['del']
        client.delete_domain(name)

    doms = []
    for d in client.domains().json():
        name = d['name']
        if len(name.split('.')) > 2:
            name = '{}.{}'.format(name.split('.')[-2], name.split('.')[-1])
        d['key_name'] = name
        doms.append(d)

    doms = sorted(doms, key=lambda k: k['name'])
    doms = sorted(doms, key=lambda k: k['key_name'])
    domains_list = []
    used_tags = []

    for d in doms:
        form = DomainForm(initial={'tags': d['tags']}, prefix=d['id'], tag_choices=tags_list)
        domains_list.append((d, form))
        used_tags.extend([tag for tag in d['tags'] if tag not in used_tags])

    res['uuid'] = client.me().json()['uuid']
    res['domains'] = domains_list
    res['new_domain'] = new_domain
    res['calendar'] = calendar
    res['tags'] = used_tags

    return main_render(request, 'console/domains.html', res)


@login_required
def domain(request, id):
    calendar = CalendarForm()
    res = {}
    client = CC(request.session.get('username'),
                request.session.get('password'),
                request.session.get('api_url'))

    all_tags = client.list_tags().json()
    if 'error' in all_tags:
        return logout(request)

    tags_list = client.get_tag_list()

    if request.POST:
        if 'tags' in request.POST or\
            'note' in request.POST:
            domain_form = DomainForm(data=request.POST, tag_choices=tags_list)
            if domain_form.is_valid():
                cd = domain_form.cleaned_data
                params = {}
                if 'tags' in cd:
                    params['tags'] = cd['tags']
                if 'note' in cd:
                    params['note'] = cd['note']
                client.update_domain(id, params)
    elif 'del' in request.GET:
        name = request.GET['del']
        client.delete_domain(name)

    domain = client.domain(id).json()
    form = DomainForm(tag_choices=tags_list, initial={
        'tags': domain['tags'], 'note': domain['note']})

    del domain['tags']
    del domain['note']

    res['calendar'] = calendar
    res['domain'] = domain
    res['domainform'] = form
    return main_render(request, 'console/domain.html', res)


@login_required
def tags(request):
    res = {}
    tagform = TagForm()
    client = CC(request.session.get('username'),
                request.session.get('password'),
                request.session.get('api_url'))

    if request.POST:
        tagform = TagForm(request.POST)
        if tagform.is_valid():
            cd = tagform.cleaned_data
            client.create_tag(cd['name'])
            tagform = TagForm()

    elif request.GET and 'id' in request.GET:
        id = request.GET['id']
        client.delete_tag(id)

    res['tags'] = client.list_tags().json()

    if 'error' in res['tags']:
        return logout(request)

    res['tagform'] = tagform
    return main_render(request, 'console/tags.html', res)


@login_required
def tag(request, tag):
    res = {}
    client = CC(request.session.get('username'),
                request.session.get('password'),
                request.session.get('api_url'))

    res['tagged_domains'] = client.domains(tags=[tag]).json()
    if 'error' in res['tagged_domains']:
        return logout(request)
    res['tag'] = tag
    res['calendar_domains'] = CalendarForm(auto_id='calendar-domains-%s')
    res['calendar_containers'] = CalendarForm(auto_id='calendar-containers-%s')
    res['tagged_containers'] = client.containers(tags=[tag]).json()
    return main_render(request, 'console/tag.html', res)


@login_required
def alarms(request):
    client = CC(request.session.get('username'),
                request.session.get('password'),
                request.session.get('api_url'))
    res = {}
    alarm_form = AlarmForm()
    alarms = client.alarms({'range': 100}).json()

    if 'error' in alarms:
        return logout(request)

    if request.POST:
        if 'action_filter' in request.POST:
            alarm_form = AlarmForm(request.POST)
            if alarm_form.is_valid():
                cd = alarm_form.cleaned_data
                cd['class'] = cd['class_']
                del cd['class_']
                r = client.alarms(cd)
                if r.uerror:
                    alarm_form.add_error(None, r.json()['error'])
                    alarms = ()
                else:
                    alarms = r.json()

    elif 'del-alarm' in request.GET:
            client.delete_alarm(request.GET['del-alarm'])

    for a in alarms:
        a['unix'] = datetime.fromtimestamp(a['unix'])
    res['alarm_form'] = alarm_form
    res['alarms'] = alarms
    return main_render(request, 'console/alarms.html', res)


@login_required
def latest_alarms(request):
    client = CC(request.session.get('username'),
                request.session.get('password'),
                request.session.get('api_url'))

    alarms = client.alarms({'range': 5})
    if 'error' in alarms:
        return logout(request)

    return HttpResponse(alarms.content)


@login_required
def alarm_key(request, id):
    if request.method == 'POST':
        client = CC(request.session.get('username'),
                    request.session.get('password'),
                    request.session.get('api_url'))
        r = client.create_alarm_key(id)
        return HttpResponse(r.content)
    return HttpResponseForbidden()


@login_required
def add_domain_tag(request, id):
    if request.method == 'POST' and '{}-tags'.format(id) in request.POST:
        client = CC(request.session.get('username'),
                    request.session.get('password'),
                    request.session.get('api_url'))
        tags_post = request.POST.getlist('{}-tags'.format(id))
        r = client.update_domain(id, {'tags': tags_post})
        return HttpResponse(r.content)
    return HttpResponseForbidden()


@login_required
def reboot_container(request, id):
    if request.method == 'POST':
        client = CC(request.session.get('username'),
                    request.session.get('password'),
                    request.session.get('api_url'))
        r = client.reboot_container(id)
        return HttpResponse(r.content)
    return HttpResponseForbidden()



from __future__ import unicode_literals


VERSION = (0, 3, 2)

__version__ = '.'.join((str(each) for each in VERSION[:4]))

DJANGO_VERSION = '==1.7'
SELECT2_VERSION = '==0.10'
UWSGIIT_VERSION = '>=0.9.1'
APPCONF_VERSION = '==0.6'


def get_version():
    """
    Returns string with digit parts only as version.

    """
    return '.'.join((str(each) for each in VERSION[:3]))



from __future__ import unicode_literals, absolute_import

from django import template


register = template.Library()


@register.filter(is_safe=True)
def get_title(value):
    value = value.title()
    return value.replace('_', ' ')


@register.tag
def get_metrics_list(parser, token):
    bits = list(token.split_contents())
    if len(bits) == 4 and bits[2] == "as":
        varname = bits[-1]
        metric_type = bits[1]
    else:
        raise template.TemplateSyntaxError("{} expected format is 'metric_type as varname'".format(bits[0]))
    if metric_type == 'domain_per_tag':
        metric_list = (
            ('console_domain_net_rx_per_tag', 'Network RX'),
            ('console_domain_net_tx_per_tag', 'Network TX'),
            ('console_domain_hits_per_tag', 'Hits')
        )
    elif metric_type == 'container_per_tag':
        metric_list = (
            ('console_container_io_read_per_tag', 'IO Read'),
            ('console_container_io_write_per_tag', 'IO Write'),
            ('console_container_net_rx_per_tag', 'Network RX'),
            ('console_container_net_tx_per_tag', 'Network TX'),
            ('console_container_cpu_per_tag', 'CPU Ticks'),
            ('console_container_mem_per_tag', 'Memory'),
            ('console_container_quota_per_tag', 'Quota')
        )
    elif metric_type == 'domain':
        metric_list = (
            ('console_domain_net_rx', 'Network RX'),
            ('console_domain_net_tx', 'Network TX'),
            ('console_domain_hits', 'Hits')
        )
    elif metric_type == 'container':
        metric_list = (
            ('console_container_io_read', 'IO Read'),
            ('console_container_io_write', 'IO Write'),
            ('console_container_net_rx', 'Network RX'),
            ('console_container_net_tx', 'Network TX'),
            ('console_container_cpu', 'CPU Ticks'),
            ('console_container_mem', 'Memory'),
            ('console_container_quota', 'Quota')
        )
    else:
        raise template.TemplateSyntaxError(
            "Expected metric types are domain_per_tag, container_per_tag, domain or container_per_tag, got {}".format(metric_type))

    return MakeListNode(metric_list, varname)


class MakeListNode(template.Node):
    def __init__(self, items, varname):
        self.items = items
        self.varname = varname

    def render(self, context):
        context[self.varname] = self.items
        return ''



from __future__ import unicode_literals, absolute_import
from datetime import date, timedelta
from calendar import monthrange

from django.utils.six.moves import xrange

from uwsgiit.api import UwsgiItClient as UC


class ConsoleClient(UC):
    def get_tag_list(self):
        return [(x['name'], x['name']) for x in self.list_tags().json()]


def diff(a, b):
    return list(set(a) - set(b))


def all_days_of(year, month=None, day=None):
        res = []
        if day and month:
            res.append(date(year, month, day))
        elif month:
            today = date.today()
            if year == today.year and month == today.month:
                last_day = today.day
            else:
                last_day = monthrange(year, month)[1]

            for d in xrange(last_day):
                res.extend(all_days_of(year, month, d+1))
        else:
            today = date.today()

            last_month = 12 if year != today.year else today.month

            for m in xrange(1, last_month+1):
                res.extend(all_days_of(year, m))
        return res


def excluded_days(all_days, days_to_exclude):
    return diff(all_days, days_to_exclude)


def daterange(start_date, end_date):
    for n in xrange((end_date - start_date).days + 1):
        yield start_date + timedelta(n)



from __future__ import unicode_literals, absolute_import
import json
from datetime import date

from django.db import models
from .utils import all_days_of, excluded_days
from . import conf


def date_from_params(params):

    year = date.today().year
    month = None
    day = None
    if 'year' not in params:
        params['year'] = year
    if 'month' in params:
        month = params['month']
    if 'day' in params:
        day = params['day']

    return params, year, month, day


class UwsgiItApi(models.Model):
    url = models.CharField(max_length=512, unique=True)
    name = models.CharField(max_length=255, blank=True)

    def __unicode__(self):
        return self.url


class GenericMetric(models.Model):
    container = models.PositiveIntegerField()

    year = models.PositiveIntegerField(null=True)
    month = models.PositiveIntegerField(null=True)
    day = models.PositiveIntegerField(null=True)

    # this ia blob containing raw metrics
    json = models.TextField(null=True)

    def __unicode__(self):
        return '{year}-{month}-{day}'.format(
            year=self.year, month=self.month, day=self.day)

    class Meta:
        abstract = True

    @property
    def unit_of_measure(self):
        return 'bytes'


class ContainerMetric(GenericMetric):

    def metrics(self, client, params={}):
        if not params:  #Gets Today metrics
            return self.api_metrics(client, params)

        params, year, month, day = date_from_params(params)

        model = self.__class__

        if day and month:
            try:
                metric = model.objects.get(container=self.container, **params)
                result = json.loads(metric.json)
            except model.DoesNotExist:
                result = self.api_metrics(client, params)
                item = model(container=self.container, json=result, **params)
                if date(year, month, day) != date.today():
                    item.save()
            return result

        results = []
        to_save = []
        items = model.objects.filter(container=self.container, **params)
        for i in items:
            results.extend(json.loads(i.json))
        all_days = all_days_of(**params)
        dates_in_db = [date(d.year, d.month, d.day) for d in items]
        for d in excluded_days(all_days, dates_in_db):
            if not month:
                params['month'] = d.month
            params['day'] = d.day
            result = self.api_metrics(client, params)
            item = model(container=self.container, json=result, **params)
            if date(params['year'], params['month'], params['day']) != date.today():
                to_save.append(item)
            results.extend(result)
        model.objects.bulk_create(to_save)
        return results

    class Meta:
        abstract = True
        unique_together = ('container', 'year', 'month', 'day')


class DomainMetric(GenericMetric):
    domain = models.PositiveIntegerField()

    def metrics(self, client, params={}, container=None):
        if not params:  #Gets Today metrics
            results = []
            for elem in self.api_metrics(client, params):
                if container:
                    if elem['container'] == container:
                        return elem['metrics']
                else:
                    results.extend(elem['metrics'])
                return results

        res = []
        to_save = []

        params, year, month, day = date_from_params(params)

        query_params = params.copy()

        if container:
            query_params['container'] = container

        model = self.__class__

        results = model.objects.filter(domain=self.domain, **query_params)

        for result in results:
            res.extend(json.loads(result.json))

        if day and month:
            if not results.exists():
                results = self.api_metrics(client, params)
                for elem in results:
                    item = model(domain=self.domain,
                                 container=elem['container'],
                                 json=elem['metrics'],
                                 **params)
                    if date(params['year'], params['month'], params['day']) != date.today():
                        to_save.append(item)
                    if not container or container == item.container:
                        res.extend(elem['metrics'])
        else:
            all_days = all_days_of(**params)
            dates_in_db = [date(d.year, d.month, d.day) for d in results]
            for d in excluded_days(all_days, dates_in_db):
                if not month:
                    params['month'] = d.month
                params['day'] = d.day
                results = self.api_metrics(client, params)
                for elem in results:
                    item = model(domain=self.domain,
                                 container=elem['container'],
                                 json=elem['metrics'],
                                 **params)
                    if date(params['year'], params['month'], params['day']) != date.today():
                        to_save.append(item)
                    if not container or container == item.container:
                            res.extend(elem['metrics'])
        model.objects.bulk_create(to_save)
        return res

    class Meta:
        abstract = True
        unique_together = ('domain', 'container', 'year', 'month', 'day')


# stores values from the tuntap router
class NetworkRXContainerMetric(ContainerMetric):
    def api_metrics(self, client, params):
        return client.container_metric(self.container, 'net.rx', params).json()

    @property
    def verbose_name(self):
        return 'Network RX'


# stores values from the tuntap router
class NetworkTXContainerMetric(ContainerMetric):
    def api_metrics(self, client, params):
        return client.container_metric(self.container, 'net.tx', params).json()

    @property
    def verbose_name(self):
        return 'Network TX'


# stores values from the container cgroup
class CPUContainerMetric(ContainerMetric):
    def api_metrics(self, client, params):
        return client.container_metric(self.container, 'cpu', params).json()

    @property
    def unit_of_measure(self):
        return 'ticks'

    @property
    def verbose_name(self):
        return 'CPU Ticks'


# stores values from the container cgroup
class MemoryContainerMetric(ContainerMetric):
    def api_metrics(self, client, params):
        return client.container_metric(self.container, 'mem', params).json()

    @property
    def verbose_name(self):
        return 'Memory'


# stores values from the container cgroup
class IOReadContainerMetric(ContainerMetric):
    def api_metrics(self, client, params):
        return client.container_metric(self.container, 'io.read', params).json()

    @property
    def verbose_name(self):
        return 'IO Read'


# stores values from the container cgroup
class IOWriteContainerMetric(ContainerMetric):
    def api_metrics(self, client, params):
        return client.container_metric(self.container, 'io.write', params).json()

    @property
    def verbose_name(self):
        return 'IO Write'


# uses perl Quota package
class QuotaContainerMetric(ContainerMetric):
    def api_metrics(self, client, params):
        return client.container_metric(self.container, 'quota', params).json()

    @property
    def verbose_name(self):
        return 'Used Disk Space'


class HitsDomainMetric(DomainMetric):
    def api_metrics(self, client, params):
        return client.domain_metric(self.domain, 'hits', params).json()

    @property
    def unit_of_measure(self):
        return 'hits'

    @property
    def verbose_name(self):
        return 'Hits'


class NetworkRXDomainMetric(DomainMetric):
    def api_metrics(self, client, params):
        return client.domain_metric(self.domain, 'net.rx', params).json()

    @property
    def verbose_name(self):
        return 'Network RX'


class NetworkTXDomainMetric(DomainMetric):
    def api_metrics(self, client, params):
        return client.domain_metric(self.domain, 'net.tx', params).json()

    @property
    def verbose_name(self):
        return 'Network TX'



from __future__ import unicode_literals, absolute_import
import json
import re

from django.http import HttpResponse
from django.forms import ValidationError
from django.views.decorators.csrf import ensure_csrf_cookie

from uwsgiit.api import UwsgiItClient as UC

from .forms import CalendarForm, MetricDetailForm
from .decorators import login_required
from .views import main_render


def stats_render(request, metrics, **kwargs):
    client = UC(request.session.get('username'),
                request.session.get('password'),
                request.session.get('api_url'))

    time_unit = 'hour'
    metric_name = 'Invalid date'
    stats = []
    if request.POST and request.is_ajax:
        calendar = CalendarForm(request.POST)
        if calendar.is_valid():
            params = calendar.get_params()
            metric_name = calendar.metric_name()
            time_unit = calendar.time_unit()
            stats = [metric.metrics(client, params) for metric in metrics]

    v_dict = {}
    v_dict['stats'] = stats
    v_dict['time_unit'] = time_unit
    v_dict['metric_name'] = metric_name
    v_dict['unit_of_measure'] = metrics[0].unit_of_measure
    v_dict['absolute_values'] = kwargs['absolute_values']
    v_dict['average'] = kwargs['average']
    return HttpResponse(json.dumps(v_dict))


@login_required
def container_metrics(request, container, **kwargs):
    metrics = [kwargs['model'](container=container)]
    return stats_render(request, metrics, **kwargs)


@login_required
def domain_metrics(request, domain, **kwargs):
    metrics = [kwargs['model'](domain=domain)]
    return stats_render(request, metrics, **kwargs)


@login_required
def container_metrics_per_tag(request, tag, **kwargs):
    client = UC(request.session.get('username'),
                request.session.get('password'),
                request.session.get('api_url'))

    containers = client.containers(tags=[tag]).json()
    metrics = [kwargs['model'](container=c['uid']) for c in containers]

    return stats_render(request, metrics, **kwargs)


@login_required
def domain_metrics_per_tag(request, tag, **kwargs):
    client = UC(request.session.get('username'),
                request.session.get('password'),
                request.session.get('api_url'))

    domains = client.domains(tags=[tag]).json()
    metrics = [kwargs['model'](domain=d['id']) for d in domains]

    return stats_render(request, metrics, **kwargs)


@login_required
@ensure_csrf_cookie
def metric_detail(request):
    v_dict = {}
    if request.GET and 'date_list[]' in request.GET:
        dates = request.GET.getlist('date_list[]')
        for date in dates:
            result = re.search(
                r'^year=\d{4}&month=\d{0,2}&day=\d{0,2}$', date)
            if result is None:
                msg = 'Invalid Date'
                raise ValidationError(msg)

        form = MetricDetailForm(request.GET)
        if form.is_valid():
            cd = form.cleaned_data
            v_dict['dates'] = json.dumps(dates)
            v_dict['url'] = cd['metric_url']
            v_dict['metric_type'] = cd['metric_type']
            v_dict['subject'] = cd['subject']
    return main_render(request, 'console/metric_detail.html', v_dict)



import random
import json
from datetime import date, timedelta

from django.test import TestCase
from django.conf import settings
from django.utils.six.moves import xrange


from uwsgiit.api import UwsgiItClient

from console.utils import daterange
from console.models import *


class MetricTesterMixin():

    @classmethod
    def istanceUwsgiItClient(cls):
        cls.client = UwsgiItClient(
            settings.TEST_USER,
            settings.TEST_PASSWORD,
            settings.DEFAULT_API_URL)

    @classmethod
    def createTestMetrics(cls, metric_class):
        parameters = {'container': 1}

        if issubclass(metric_class, ContainerMetric):
            pass
        elif issubclass(metric_class, DomainMetric):
            parameters['domain'] = 1
        else:
            raise TypeError('Cannot handle {class_name} class'.format(
                class_name=metric_class.__name__))

        start = date(2010, 1, 1)
        end = date(2010, 12, 31)

        cls.test_metrics = []

        for day in daterange(start, end):
            json = []
            for i in xrange(5):
                json.append([random.randint(1, 100), random.randint(1, 100)])
            parameters['day'] = day.day
            parameters['month'] = day.month
            parameters['year'] = day.year
            parameters['json'] = json
            test_metric = metric_class(**parameters)
            test_metric.save()
            cls.test_metrics.append(test_metric)

    @classmethod
    def destroyTestMetrics(cls):
        for metric in cls.test_metrics:
            metric.delete()


class ContainerMetricTests(MetricTesterMixin, TestCase):

    @classmethod
    def setUpClass(cls):
        cls.createTestMetrics(NetworkRXContainerMetric)
        cls.today = date.today()
        cls.yesterday = cls.today - timedelta(1)
        cls.tomorrow = cls.today + timedelta(1)

    def test_IOReadContainerMetric_returns_right_unit_of_measure(self):
        self.assertEqual(IOReadContainerMetric().unit_of_measure, 'bytes')

    def test_IOWriteContainerMetric_returns_right_unit_of_measure(self):
        self.assertEqual(IOWriteContainerMetric().unit_of_measure, 'bytes')

    def test_NetworkRXContainerMetric_returns_right_unit_of_measure(self):
        self.assertEqual(NetworkRXContainerMetric().unit_of_measure, 'bytes')

    def test_NetworkTXContainerMetric_returns_right_unit_of_measure(self):
        self.assertEqual(NetworkTXContainerMetric().unit_of_measure, 'bytes')

    def test_CPUContainerMetric_returns_right_unit_of_measure(self):
        self.assertEqual(CPUContainerMetric().unit_of_measure, 'ticks')

    def test_MemoryContainerMetric_returns_right_unit_of_measure(self):
        self.assertEqual(MemoryContainerMetric().unit_of_measure, 'bytes')

    def test_QuotaContainerMetric_returns_right_unit_of_measure(self):
        self.assertEqual(QuotaContainerMetric().unit_of_measure, 'bytes')

    def test_generic_metric_to_string_prints_date(self):
        self.assertEqual(self.test_metrics[0].__unicode__(), '2010-1-1')

    def test_metrics_returns_right_json_for_specific_day_from_db(self):
        result = NetworkRXContainerMetric(container=1).metrics(
            self.client, params={'year': 2010, 'month': 1, 'day': 1})
        self.assertEqual(len(result), len(self.test_metrics[0].json))
        self.assertEqual(sorted(result), sorted(self.test_metrics[0].json))

    def test_metrics_returns_right_json_for_specific_month_from_db(self):
        results = NetworkRXContainerMetric(container=1).metrics(
            self.client, params={'year': 2010, 'month': 2})
        january_metrics = []
        [january_metrics.extend(el.json) for el in self.test_metrics[31:59]]
        self.assertEqual(len(results), len(january_metrics))
        self.assertEqual(sorted(results), sorted(january_metrics))

    def test_metrics_returns_right_json_for_specific_year_from_db(self):
        results = NetworkRXContainerMetric(container=1).metrics(
            self.client, params={'year': 2010})
        year_metrics = []
        [year_metrics.extend(el.json) for el in self.test_metrics]
        self.assertEqual(len(results), len(year_metrics))
        self.assertEqual(sorted(results), sorted(year_metrics))

    def test_metrics_does_not_save_current_day_in_db(self):
        client = UwsgiItClient(
            settings.TEST_USER,
            settings.TEST_PASSWORD,
            settings.DEFAULT_API_URL)

        NetworkRXContainerMetric(container=settings.TEST_CONTAINER).metrics(
            client, params={
                'year': self.today.year,
                'month': self.today.month,
                'day': self.today.day})

        self.assertRaises(
            NetworkRXContainerMetric.DoesNotExist,
            NetworkRXContainerMetric.objects.get,
            **{'container': settings.TEST_CONTAINER,
               'year': self.yesterday.year,
               'month': self.yesterday.month,
               'day': self.yesterday.day})

    def test_metrics_returns_right_json_for_specific_day_from_api_and_saves_in_db(self):
        client = UwsgiItClient(
            settings.TEST_USER,
            settings.TEST_PASSWORD,
            settings.DEFAULT_API_URL)

        results = NetworkRXContainerMetric(container=settings.TEST_CONTAINER).metrics(
            client, params={
                'year': self.yesterday.year,
                'month': self.yesterday.month,
                'day': self.yesterday.day})

        metric_from_db = NetworkRXContainerMetric.objects.get(
            container=settings.TEST_CONTAINER,
            year=self.yesterday.year,
            month=self.yesterday.month,
            day=self.yesterday.day)

        self.assertEqual(results, json.loads(metric_from_db.json))

    def test_metrics_does_not_return_metrics_from_future_asking_for_current_month(self):
        """Assumes that today is not the last day of month"""

        client = UwsgiItClient(
            settings.TEST_USER,
            settings.TEST_PASSWORD,
            settings.DEFAULT_API_URL)

        NetworkRXContainerMetric(container=settings.TEST_CONTAINER).metrics(
            client, params={
                'year': self.today.year,
                'month': self.today.month})

        self.assertRaises(
            NetworkRXContainerMetric.DoesNotExist,
            NetworkRXContainerMetric.objects.get,
            **{'container': settings.TEST_CONTAINER,
               'year': self.tomorrow.year,
               'month': self.tomorrow.month,
               'day': self.tomorrow.day})

    def test_metrics_returns_presents_metrics_from_db_and_missing_metrics_from_api(self):
        """Assumes that today is not the first day of month"""

        client = UwsgiItClient(
            settings.TEST_USER,
            settings.TEST_PASSWORD,
            settings.DEFAULT_API_URL)

        test_metric = NetworkRXContainerMetric(
            container=settings.TEST_CONTAINER,
            json=[[-1, -2], [-3, -4]],
            year=self.yesterday.year,
            month=self.yesterday.month,
            day=self.yesterday.day)
        test_metric.save()

        results = NetworkRXContainerMetric(container=settings.TEST_CONTAINER).metrics(
            client, params={
                'year': self.today.year,
                'month': self.today.month})

        self.assertIn(test_metric.json[0], results)
        self.assertIn(test_metric.json[1], results)

        test_metric.delete()

    @classmethod
    def tearDownClass(cls):
        cls.destroyTestMetrics()


class DomainMetricTests(MetricTesterMixin, TestCase):

    @classmethod
    def setUpClass(cls):
        cls.createTestMetrics(NetworkRXDomainMetric)
        cls.today = date.today()
        cls.yesterday = cls.today - timedelta(1)
        cls.tomorrow = cls.today + timedelta(1)

    def test_NetworkRXDomainMetric_returns_right_unit_of_measure(self):
        self.assertEqual(NetworkRXDomainMetric().unit_of_measure, 'bytes')

    def test_NetworkTXDomainMetric_returns_right_unit_of_measure(self):
        self.assertEqual(NetworkTXDomainMetric().unit_of_measure, 'bytes')

    def test_HitsDomainMetric_returns_right_unit_of_measure(self):
        self.assertEqual(HitsDomainMetric().unit_of_measure, 'hits')

    def test_generic_metric_to_string_prints_date(self):
        self.assertEqual(self.test_metrics[0].__unicode__(), '2010-1-1')

    def test_metrics_returns_right_json_for_specific_day_from_db(self):
        result = NetworkRXDomainMetric(domain=1).metrics(
            self.client, params={'year': 2010, 'month': 1, 'day': 1})
        self.assertEqual(len(result), len(self.test_metrics[0].json))
        self.assertEqual(sorted(result), sorted(self.test_metrics[0].json))

    def test_metrics_returns_right_json_for_specific_month_from_db(self):
        results = NetworkRXDomainMetric(domain=1).metrics(
            self.client, params={'year': 2010, 'month': 2})
        january_metrics = []
        [january_metrics.extend(el.json) for el in self.test_metrics[31:59]]
        self.assertEqual(len(results), len(january_metrics))
        self.assertEqual(sorted(results), sorted(january_metrics))

    def test_metrics_returns_right_json_for_specific_year_from_db(self):
        results = NetworkRXDomainMetric(domain=1).metrics(
            self.client, params={'year': 2010})
        year_metrics = []
        [year_metrics.extend(el.json) for el in self.test_metrics]
        self.assertEqual(len(results), len(year_metrics))
        self.assertEqual(sorted(results), sorted(year_metrics))

    def test_metrics_does_not_save_current_day_in_db(self):
        client = UwsgiItClient(
            settings.TEST_USER,
            settings.TEST_PASSWORD,
            settings.DEFAULT_API_URL)

        NetworkRXDomainMetric(domain=settings.TEST_DOMAIN).metrics(
            client, params={
                'year': self.today.year,
                'month': self.today.month,
                'day': self.today.day})

        self.assertRaises(
            NetworkRXDomainMetric.DoesNotExist,
            NetworkRXDomainMetric.objects.get,
            **{'domain': settings.TEST_DOMAIN,
               'year': self.yesterday.year,
               'month': self.yesterday.month,
               'day': self.yesterday.day})

    def test_metrics_returns_right_json_for_specific_day_from_api_and_saves_in_db(self):
        client = UwsgiItClient(
            settings.TEST_USER,
            settings.TEST_PASSWORD,
            settings.DEFAULT_API_URL)

        results = NetworkRXDomainMetric(domain=settings.TEST_DOMAIN).metrics(
            client, params={
                'year': self.yesterday.year,
                'month': self.yesterday.month,
                'day': self.yesterday.day})

        metric_from_db = NetworkRXDomainMetric.objects.get(
            domain=settings.TEST_DOMAIN,
            year=self.yesterday.year,
            month=self.yesterday.month,
            day=self.yesterday.day)

        self.assertEqual(results, json.loads(metric_from_db.json))

    def test_metrics_does_not_return_metrics_from_future_asking_for_current_month(self):
        """Assumes that today is not the last day of month"""

        client = UwsgiItClient(
            settings.TEST_USER,
            settings.TEST_PASSWORD,
            settings.DEFAULT_API_URL)

        NetworkRXDomainMetric(domain=settings.TEST_DOMAIN).metrics(
            client, params={
                'year': self.today.year,
                'month': self.today.month})

        self.assertRaises(
            NetworkRXDomainMetric.DoesNotExist,
            NetworkRXDomainMetric.objects.get,
            **{'domain': settings.TEST_DOMAIN,
               'year': self.tomorrow.year,
               'month': self.tomorrow.month,
               'day': self.tomorrow.day})

    def test_metrics_returns_presents_metrics_from_db_and_missing_metrics_from_api(self):
        """Assumes that today is not the first day of month"""

        client = UwsgiItClient(
            settings.TEST_USER,
            settings.TEST_PASSWORD,
            settings.DEFAULT_API_URL)

        test_metric = NetworkRXDomainMetric(
            container=settings.TEST_CONTAINER,
            domain=settings.TEST_DOMAIN,
            json=[[-1, -2], [-3, -4]],
            year=self.yesterday.year,
            month=self.yesterday.month,
            day=self.yesterday.day)
        test_metric.save()

        results = NetworkRXDomainMetric(domain=settings.TEST_DOMAIN).metrics(
            client, params={
                'year': self.today.year,
                'month': self.today.month})

        self.assertIn(test_metric.json[0], results)
        self.assertIn(test_metric.json[1], results)

        test_metric.delete()

    @classmethod
    def tearDownClass(cls):
        cls.destroyTestMetrics()



from __future__ import unicode_literals, absolute_import

from django.contrib import admin

from .models import *


class ContainerMetricAdmin(admin.ModelAdmin):
    list_display = ('__unicode__', 'container', 'year', 'month', 'day')
    list_filter = ('year', 'month', 'day')


class DomainMetricAdmin(admin.ModelAdmin):
    list_display = ('__unicode__', 'container', 'year', 'month', 'day')
    list_filter = ('year', 'month', 'day')


class UwsgiItApiAdmin(admin.ModelAdmin):
    list_display = ('__unicode__', 'name', 'url')
    list_filter = ('name', 'url')


admin.site.register(NetworkRXContainerMetric, ContainerMetricAdmin)
admin.site.register(NetworkTXContainerMetric, ContainerMetricAdmin)
admin.site.register(CPUContainerMetric, ContainerMetricAdmin)
admin.site.register(MemoryContainerMetric, ContainerMetricAdmin)
admin.site.register(IOReadContainerMetric, ContainerMetricAdmin)
admin.site.register(IOWriteContainerMetric, ContainerMetricAdmin)
admin.site.register(QuotaContainerMetric, ContainerMetricAdmin)
admin.site.register(HitsDomainMetric, DomainMetricAdmin)
admin.site.register(NetworkRXDomainMetric, DomainMetricAdmin)
admin.site.register(NetworkTXDomainMetric, DomainMetricAdmin)
admin.site.register(UwsgiItApi, UwsgiItApiAdmin)



from django.test import TestCase
from django.conf import settings

from ..models import UwsgiItApi


class ConsoleContextProcessorTests(TestCase):
    @classmethod
    def setUpClass(cls):
        cls.test_api = UwsgiItApi(
            url=settings.DEFAULT_API_URL,
            name='TEST API')
        cls.test_api.save()

    def test_request_attributes(self):
        url = '/'
        response = self.client.get(url)
        try:
            self.assertEqual(
                settings.CONSOLE_TITLE, response.context['CONSOLE_TITLE'])
            self.assertEqual(
                settings.CONSOLE_SUBTITLE, response.context['CONSOLE_SUBTITLE'])
            self.assertEqual(
                settings.JQUERY_LIBJQUERY_LIB, response.context['JQUERY_LIB'])
        except AttributeError:
            pass
        self.assertEqual(response.context['path'], url)

    @classmethod
    def tearDownClass(cls):
        cls.test_api.delete()



from __future__ import unicode_literals, absolute_import

from appconf import AppConf
from django.conf import settings


class ConsoleConf(AppConf):
    CONSOLE_TITLE = 'uWSGI.it Console'
    CONSOLE_SUBTITLE = ''
    DEFAULT_API_URL = 'https://api.uwsgi.it/api/'

    def configure_console_title(self, value):
        if not getattr(settings, 'CONSOLE_TITLE', None):
            self._meta.holder.CONSOLE_TITLE = value
            return value
        return getattr(settings, 'CONSOLE_TITLE')

    def configure_console_subtitle(self, value):
        if not getattr(settings, 'CONSOLE_SUBTITLE', None):
            self._meta.holder.CONSOLE_SUBTITLE = value
            return value
        return getattr(settings, 'CONSOLE_SUBTITLE')

    def configure_default_api_url(self, value):
        if not getattr(settings, 'DEFAULT_API_URL', None):
            self._meta.holder.DEFAULT_API_URL = value
            return value
        return getattr(settings, 'DEFAULT_API_URL')



import unittest
from datetime import date

from django.utils.six.moves import xrange

from console.utils import all_days_of, excluded_days, daterange


class TestUtils(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        start = date(2014, 1, 1)
        end = date(2014, 1, 31)
        cls.all_days_of_january = list(daterange(start, end))
        end = date(2014, 1, 30)
        cls.some_days_of_january = list(daterange(start, end))
        cls.missing_days_of_january = [date(2014, 1, 31)]
        cls.today = date.today()

    def test_daterange_generate_right_values(self):
        self.assertEqual(len(self.all_days_of_january), 31)
        self.assertEqual(len(self.some_days_of_january), 30)
        for i in xrange(1, 32):
            day = date(2014, 1, i)
            self.assertIn(day, self.all_days_of_january)

    def test_excluded_days_returns_missing_day(self):
        results = excluded_days(self.all_days_of_january, self.some_days_of_january)
        self.assertEqual(results, self.missing_days_of_january)

    def test_excluded_days_returns_empty_list(self):
        results = excluded_days(self.all_days_of_january, self.all_days_of_january)
        self.assertEqual(results, [])

    def test_excluded_days_returns_multiple_missing_days(self):
        sixth_january = self.some_days_of_january.pop(5)
        self.missing_days_of_january.append(sixth_january)
        results = excluded_days(self.all_days_of_january, self.some_days_of_january)
        self.assertEqual(sorted(results), sorted(self.missing_days_of_january))
        self.missing_days_of_january.remove(sixth_january)
        self.some_days_of_january.append(sixth_january)

    def test_all_days_of_returns_all_days_of_an_year(self):
        results = all_days_of(2010, month=None, day=None)
        self.assertEqual(len(results), 365)
        self.assertEqual(results[0], date(2010, 1, 1))
        self.assertEqual(results[-1], date(2010, 12, 31))

    def test_all_days_of_returns_all_days_of_a_month(self):
        results = all_days_of(2014, month=1, day=None)
        self.assertEqual(len(results), 31)
        self.assertEqual(results, self.all_days_of_january)

    def test_all_days_of_year_returns_one_day(self):
        results = all_days_of(2014, month=1, day=1)
        self.assertEqual(len(results), 1)
        self.assertEqual(results[0], self.all_days_of_january[0])

    def test_all_days_handles_current_month(self):
        results = all_days_of(date.today().year, self.today.month)
        first_of_month = date(self.today.year, self.today.month, 1)
        all_days_of_the_month_untill_today = list(daterange(first_of_month, self.today))
        self.assertEqual(results, all_days_of_the_month_untill_today)

    def test_all_days_handles_current_year(self):
        results = all_days_of(date.today().year)
        first_of_year = date(self.today.year, 1, 1)
        all_days_of_the_year_untill_today = list(daterange(first_of_year, self.today))
        self.assertEqual(results, all_days_of_the_year_untill_today)



from __future__ import unicode_literals, absolute_import
from datetime import datetime, timedelta

from django import forms
from django.conf import settings
from django.utils.dates import MONTHS
from django.core.validators import validate_email
from django.core.urlresolvers import resolve, Resolver404

from uwsgiit.api import UwsgiItClient
from select2.widgets import SelectMultipleAutocomplete, SelectAutocomplete

from .models import UwsgiItApi


def email_list_validator(value):
    "Check if value consists only of valid emails."
    # Use the parent's handling of required fields, etc.
    for email in value:
        validate_email(email.strip())


class MultiEmailField(forms.CharField):

    default_validators = [email_list_validator]

    def to_python(self, value):
        "Normalize data to a list of strings."
        # Return an empty list if no input was given.
        if value in self.empty_values:
            return []
        return value.split(',')

    def clean(self, value):
        value = super(MultiEmailField, self).clean(value)
        return ','.join([email.strip() for email in value])


class TagsForm(forms.Form):
    tags = forms.MultipleChoiceField(
        widget=SelectMultipleAutocomplete(plugin_options={"width": "300px"}),
        choices=(),
        required=False)

    def __init__(self, *args, **kwargs):
        tag_choices = kwargs.pop('tag_choices')
        super(TagsForm, self).__init__(*args, **kwargs)
        self.fields['tags'].choices = tag_choices


class BootstrapForm(forms.Form):

    def __init__(self, *args, **kwargs):
        super(BootstrapForm, self).__init__(*args, **kwargs)
        for field in self.fields.keys():
            if not isinstance(self.fields[field].widget, (SelectAutocomplete, SelectMultipleAutocomplete)):
                self.fields[field].widget.attrs['class'] = 'form-control'


class LoginForm(forms.Form):
    action_login = forms.IntegerField(
        label='', widget=forms.HiddenInput(), initial=1)
    username = forms.CharField(label='', widget=forms.TextInput(
        attrs={'class': 'form-control', 'placeholder': 'Username'}))
    password = forms.CharField(label='', widget=forms.PasswordInput(
        attrs={'class': 'form-control', 'placeholder': 'Password'}))
    api_url = forms.ModelChoiceField(
        label='Api url :', queryset=UwsgiItApi.objects.none())

    def __init__(self, *args, **kwargs):
        super(LoginForm, self).__init__(*args, **kwargs)
        self.fields['api_url'].queryset = UwsgiItApi.objects.all()
        self.fields['api_url'].initial = UwsgiItApi.objects.get(
            url=settings.DEFAULT_API_URL)

    def clean(self):
        cd = super(LoginForm, self).clean()
        if 'username' in cd and 'password' in cd and 'api_url' in cd:
            client = UwsgiItClient(
                cd['username'],
                cd['password'],
                cd['api_url'].url)

            me = client.me().json()
            if 'error' in me:
                raise forms.ValidationError('Wrong username or password')
        return cd


class MeForm(forms.Form):
    company = forms.CharField(label='Company', widget=forms.TextInput(
        attrs={'class': 'form-control col-xs-8'}))
    email = MultiEmailField(label='Email', widget=forms.TextInput(
        attrs={'class': 'form-control col-xs-8'}), required=False)
    password = forms.CharField(label='Password', widget=forms.PasswordInput(
        attrs={'class': 'form-control'}, render_value=True))
    re_password = forms.CharField(
        label='Retype password',
        widget=forms.PasswordInput(
            render_value=True, attrs={'class': 'form-control'}))
    vat = forms.CharField(label='Vat', widget=forms.TextInput(
        attrs={'class': 'form-control col-xs-8'}), required=False)

    def clean(self):
        cd = super(MeForm, self).clean()
        if 'password' in cd and 're_password' in cd:
            p1 = cd['password']
            p2 = cd['re_password']
            if p1 != p2:
                self._errors['re_password'] = self.error_class(
                    ['Passwords do not match'])
        return cd


class SSHForm(forms.Form):
    key = forms.CharField(label='ssh key', widget=forms.Textarea(
        attrs={'cols': 100, 'rows': 3, 'class': 'form-control'}))

    def clean(self):
        """Raise a ValidationError if the
           value is not bigger than 130 bytes
           check for ssh-rsa and ssh-dsa strings
        """
        data = super(SSHForm, self).clean()
        if 'key' in data:
            key = data['key'].strip()
            if len(key) > 130:
                if not 'ssh-rsa ' in key and not 'ssh-dsa ' in key:
                    msg = 'Inserted value is not a valid ssh key'
                    raise forms.ValidationError(msg)
                if key.count('\n') > 0:
                    msg = 'Too many newlines in the ssh key'
                    raise forms.ValidationError(msg)
            else:
                msg = 'Key too short'
                raise forms.ValidationError(msg)
        return data


class ContainerForm(TagsForm):
    name = forms.CharField(label='Name', required=False)
    quota_threshold = forms.IntegerField(
        label='Quota Threshold', min_value=0, max_value=100)
    nofollow = forms.BooleanField(label='NoFollow', required=False)
    distro = forms.CharField(label='Distro', widget=forms.Select(choices=()))
    linked_to = forms.MultipleChoiceField(
        widget=SelectMultipleAutocomplete(plugin_options={"width": "300px"}),
        choices=(),
        required=False)
    jid = forms.CharField(label='Jabber ID', required=False)
    jid_destinations = forms.CharField(
        label='Jabber Destinations', required=False)
    jid_secret = forms.CharField(
        label='Jabber Password', widget=forms.PasswordInput(), required=False)

    pushover_user = forms.CharField(label='Pushover User', required=False)
    pushover_token = forms.CharField(label='Pushover Token', required=False)
    pushover_sound = forms.CharField(label='Pushover Sound', required=False)

    alarm_freq = forms.IntegerField(
        label='Alarm Frequency', required=False, min_value=60)

    note = forms.CharField(
        widget=forms.Textarea(
            attrs={'cols': 50, 'rows': 3, 'class': 'form-control'}),
        required=False)

    reboot = forms.BooleanField(required=False, widget=forms.HiddenInput)

    def __init__(self, *args, **kwargs):
        distro_choices = kwargs.pop('distro_choices')
        linked_to_choices = kwargs.pop('linked_to_choices')
        super(ContainerForm, self).__init__(*args, **kwargs)
        self.fields['distro'].widget.choices = distro_choices
        self.fields['linked_to'].choices = linked_to_choices


class TagForm(forms.Form):
    name = forms.CharField(label='Name')


class DomainForm(TagsForm):
    note = forms.CharField(required=False, widget=forms.Textarea(
        attrs={'cols': 50, 'rows': 3, 'class': 'form-control'}))


class NewDomainForm(forms.Form):
    name = forms.CharField(
        label='Name', widget=forms.TextInput(attrs={'size': 70}))


class CalendarForm(forms.Form):
    year = forms.IntegerField()
    month = forms.ChoiceField(
        required=False,
        widget=SelectAutocomplete(plugin_options={"width": "200px"}),
        choices=[('', '')] + [(k, v) for k, v in MONTHS.items()])
    day = forms.IntegerField(required=False)

    def __init__(self, *args, **kwargs):
        super(CalendarForm, self).__init__(*args, **kwargs)
        today = datetime.today()
        yesterday = today - timedelta(1)
        self.fields['year'].initial = yesterday.year
        self.fields['month'].initial = yesterday.month
        self.fields['day'].initial = yesterday.day
        self.fields['day'].widget.attrs['min'] = 1

    def has_value(self, field):
        data = self.cleaned_data
        if field in data and data[field]:
            return True
        return False

    def get_params(self):
        res = {}
        data = self.cleaned_data
        if self.has_value('year'):
            res['year'] = data['year']
        if self.has_value('month'):
            res['month'] = int(data['month'])
        if self.has_value('day'):
            res['day'] = data['day']
        return res

    def metric_name(self):
        metric_name = ''
        data = self.cleaned_data
        if self.has_value('year'):
            metric_name = str(data['year'])
            if self.has_value('month'):
                metric_name = str(data['month']) + '-' + metric_name
                if self.has_value('day'):
                    metric_name = str(data['day']) + '-' + metric_name
        return metric_name

    def time_unit(self):
        if self.has_value('day'):
            return 'hour'
        elif self.has_value('month'):
            return 'day'
        return 'month'

    def is_in_the_future(self):
        data = self.get_params()
        today = datetime.today()
        if 'year' in data and data['year'] > today.year:
            return True
        if ('year' in data and data['year'] == today.year and
           'month' in data and data['month'] > today.month):
            return True
        if ('year' in data and data['year'] == today.year and
           'month' in data and data['month'] == today.month and
           'day' in data and data['day'] > today.day):
            return True
        return False

    def clean(self):
        data = super(CalendarForm, self).clean()
        if self.has_value('day') and not self.has_value('month'):
            self._errors['month'] = self.error_class(['Month is required.'])
        if self.is_in_the_future():
            raise forms.ValidationError('Set a date in the past.')
        return data


class MetricDetailForm(forms.Form):
    metric_url = forms.CharField()
    metric_type = forms.CharField()
    subject = forms.CharField()

    def clean(self):
        cd = super(MetricDetailForm, self).clean()
        if 'metric_url' in cd:
            try:
                resolve(cd['metric_url'])
            except Resolver404:
                raise forms.ValidationError('Invalid url')
        return cd


class NewLoopboxForm(BootstrapForm):
    # container = forms.IntegerField(label='', widget=forms.HiddenInput())
    filename = forms.CharField(label='Filename')
    mountpoint = forms.CharField(label='Mount Point')
    readonly = forms.BooleanField(label='Readonly', required=False)


class LoopboxForm(TagsForm):
    lid = forms.IntegerField(widget=forms.HiddenInput, required=False)


class AlarmForm(BootstrapForm):
    action_filter = forms.IntegerField(
        label='', widget=forms.HiddenInput(), initial=1)
    container = forms.IntegerField(required=False)
    vassal = forms.CharField(required=False)
    class_ = forms.CharField(label='Class', required=False)
    color = forms.CharField(max_length=7, required=False)
    level = forms.ChoiceField(
        required=False,
        widget=SelectAutocomplete(plugin_options={"width": "100%"}),
        choices=(
            ('', ' '), (0, 'System'), (1, 'User'),
            (2, 'Exception'), (3, 'Traceback'), (4, 'Log')
        )
    )
    line = forms.IntegerField(min_value=0, required=False)
    filename = forms.CharField(required=False)
    func = forms.CharField(label='Function', required=False)

    def clean(self):
        cd = super(AlarmForm, self).clean()
        del cd['action_filter']
        return cd



from __future__ import unicode_literals, absolute_import

try:
    from django.conf.urls import patterns, url
except ImportError:
    from django.conf.urls.defaults import patterns, url

from .models import IOReadContainerMetric, IOWriteContainerMetric,\
    NetworkRXContainerMetric, NetworkTXContainerMetric, CPUContainerMetric,\
    MemoryContainerMetric, QuotaContainerMetric, NetworkRXDomainMetric,\
    NetworkTXDomainMetric, HitsDomainMetric


urlpatterns = patterns(
    'console.views',
    url(r'^$', 'home', name='console_home'),
    url(r'^me/$', 'me_page', name='console_me'),
    url(r'^logout/$', 'logout', name='console_logout'),
    url(r'^containers/(?P<id>\d+)$', 'containers', name='console_containers'),
    url(r'^domains/(?P<id>\d+)$', 'domain', name='console_domain'),
    url(r'^domains/$', 'domains', name='console_domains'),
    url(r'^tags/(?P<tag>.+)$', 'tag', name='console_tag'),
    url(r'^tags/$', 'tags', name='console_tags'),
    url(r'^alarms/$', 'alarms', name='console_alarms'),
    url(r'^latest_alarms/$', 'latest_alarms', name='console_latest_alarms'),
    url(r'^alarm_key/(?P<id>\d+)$', 'alarm_key', name='console_alarm_key'),
    url(r'^add_domain_tag/(?P<id>\d+)$', 'add_domain_tag', name='console_add_domain_tag'),
    url(r'^reboot_container/(?P<id>\d+)$', 'reboot_container', name='console_reboot_container'),
)

urlpatterns += patterns(
    'console.views_metrics',
    url(r'^metric/$', 'metric_detail', name='console_metric_detail'),
    url(r'^metrics/container/io.read/id/(\d+)/$', 'container_metrics', name='console_container_io_read',
        kwargs={'model': IOReadContainerMetric, 'absolute_values': False, 'average': False}),
    url(r'^metrics/container/io.write/id/(\d+)/$', 'container_metrics', name='console_container_io_write',
        kwargs={'model': IOWriteContainerMetric, 'absolute_values': False, 'average': False}),
    url(r'^metrics/container/net.rx/id/(\d+)/$', 'container_metrics', name='console_container_net_rx',
        kwargs={'model': NetworkRXContainerMetric, 'absolute_values': False, 'average': False}),
    url(r'^metrics/container/net.tx/id/(\d+)/$', 'container_metrics', name='console_container_net_tx',
        kwargs={'model': NetworkTXContainerMetric, 'absolute_values': False, 'average': False}),
    url(r'^metrics/container/cpu/id/(\d+)/$', 'container_metrics', name='console_container_cpu',
        kwargs={'model': CPUContainerMetric, 'absolute_values': False, 'average': False}),
    url(r'^metrics/container/mem/id/(\d+)/$', 'container_metrics', name='console_container_mem',
        kwargs={'model': MemoryContainerMetric, 'absolute_values': True, 'average': True}),
    url(r'^metrics/container/quota/id/(\d+)/$', 'container_metrics', name='console_container_quota',
        kwargs={'model': QuotaContainerMetric, 'absolute_values': True, 'average': False}),

    url(r'^metrics/container/io.read/tag/(.+)/$', 'container_metrics_per_tag', name='console_container_io_read_per_tag',
        kwargs={'model': IOReadContainerMetric, 'absolute_values': False, 'average': False}),
    url(r'^metrics/container/io.write/tag/(.+)/$', 'container_metrics_per_tag', name='console_container_io_write_per_tag',
        kwargs={'model': IOWriteContainerMetric, 'absolute_values': False, 'average': False}),
    url(r'^metrics/container/net.rx/tag/(.+)/$', 'container_metrics_per_tag', name='console_container_net_rx_per_tag',
        kwargs={'model': NetworkRXContainerMetric, 'absolute_values': False, 'average': False}),
    url(r'^metrics/container/net.tx/tag/(.+)/$', 'container_metrics_per_tag', name='console_container_net_tx_per_tag',
        kwargs={'model': NetworkTXContainerMetric, 'absolute_values': False, 'average': False}),
    url(r'^metrics/container/cpu/tag/(.+)/$', 'container_metrics_per_tag', name='console_container_cpu_per_tag',
        kwargs={'model': CPUContainerMetric, 'absolute_values': False, 'average': True}),
    url(r'^metrics/container/mem/tag/(.+)/$', 'container_metrics_per_tag', name='console_container_mem_per_tag',
        kwargs={'model': MemoryContainerMetric, 'absolute_values': True, 'average': True}),
    url(r'^metrics/container/quota/tag/(.+)/$', 'container_metrics_per_tag', name='console_container_quota_per_tag',
        kwargs={'model': QuotaContainerMetric, 'absolute_values': True, 'average': False}),

    url(r'^metrics/domain/net.rx/id/(\d+)/$', 'domain_metrics', name='console_domain_net_rx',
        kwargs={'model': NetworkRXDomainMetric, 'absolute_values': False, 'average': False}),
    url(r'^metrics/domain/net.tx/id/(\d+)/$', 'domain_metrics', name='console_domain_net_tx',
        kwargs={'model': NetworkTXDomainMetric, 'absolute_values': False, 'average': False}),
    url(r'^metrics/domain/hits/id/(\d+)/$', 'domain_metrics', name='console_domain_hits',
        kwargs={'model': HitsDomainMetric, 'absolute_values': False, 'average': False}),

    url(r'^metrics/domain/net.rx/tag/(.+)/$', 'domain_metrics_per_tag', name='console_domain_net_rx_per_tag',
        kwargs={'model': NetworkRXDomainMetric, 'absolute_values': False, 'average': False}),
    url(r'^metrics/domain/net.tx/tag/(.+)/$', 'domain_metrics_per_tag', name='console_domain_net_tx_per_tag',
        kwargs={'model': NetworkTXDomainMetric, 'absolute_values': False, 'average': False}),
    url(r'^metrics/domain/hits/tag/(.+)/$', 'domain_metrics_per_tag', name='console_domain_hits_per_tag',
        kwargs={'model': HitsDomainMetric, 'absolute_values': False, 'average': False}),
)



from __future__ import unicode_literals, absolute_import

from django.conf import settings


def console_context(request):
    context_extras = {}
    try:
        context_extras['CONSOLE_TITLE'] = settings.CONSOLE_TITLE
        context_extras['CONSOLE_SUBTITLE'] = settings.CONSOLE_SUBTITLE
        context_extras['JQUERY_LIB'] = settings.JQUERY_LIBJQUERY_LIB
    except AttributeError:
        pass
    context_extras['path'] = request.get_full_path()
    return context_extras



from setuptools import setup, find_packages
import os
import console

setup(
    name='uwsgiit-console',
    version=console.__version__,
    description='A django app to simplify the use of uWSGI.it API',
    long_description=open(
        os.path.join(os.path.dirname(__file__), 'README.md')).read(),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: JavaScript",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    keywords='uwsgit.it,console',
    author='20tab S.r.l.',
    author_email='info@20tab.com',
    url='https://github.com/20tab/uwsgiit_console',
    license='MIT License',
    platforms=['OS Independent'],
    install_requires=[
        'Django{}'.format(console.DJANGO_VERSION),
        'twentytab-select2{}'.format(console.SELECT2_VERSION),
        'django-appconf{}'.format(console.APPCONF_VERSION),
        'uwsgiit-client{}'.format(console.UWSGIIT_VERSION)
    ],
    packages=find_packages(exclude=['demo', 'demo.*']),
    include_package_data=True,
    package_data={
        '': ['*.html', '*.css', '*.js', '*.gif', '*.png', ],
    }
)



from django.test import TestCase, RequestFactory
from django.core.urlresolvers import resolve, reverse
from django.http import HttpResponseRedirect
from django.conf import settings

from console.forms import NewDomainForm, TagForm
from console.views import home, me_page, logout, domains, domain, tags,\
    tag, containers, alarms, latest_alarms
from console.views_metrics import container_metrics, domain_metrics,\
    container_metrics_per_tag, domain_metrics_per_tag
from console.models import IOReadContainerMetric, NetworkRXDomainMetric,\
    UwsgiItApi


class HomeViewTests(TestCase):

    @classmethod
    def setUpClass(cls):
        cls.test_api = UwsgiItApi(
            url=settings.DEFAULT_API_URL,
            name='TEST API')
        cls.test_api.save()

        request_factory = RequestFactory()
        cls.request = request_factory.get('/', follow=True)
        cls.request.session = {}
        cls.request_post = request_factory.post('/', follow=True, data={
            'username': settings.TEST_USER,
            'password': settings.TEST_PASSWORD,
            'api_url': cls.test_api.id,
            'action_login': 1})
        cls.request_post.session = {}

    @classmethod
    def tearDownClass(cls):
        cls.test_api.delete()

    def test_home_name_resolves_to_home_url(self):
        url = reverse('console_home')
        self.assertEqual(url, '/')

    def test_home_url_resolves_to_home_view(self):
        resolver = resolve('/')
        self.assertEqual(resolver.func, home)

    def test_home_returns_appropriate_html_respons_code(self):
        response = home(self.request)
        self.assertEqual(response.status_code, 200)

    def test_home_contains_right_html(self):
        response = home(self.request)
        self.assertContains(response, 'id="id_action_login" name="action_login"')
        self.assertNotContains(response, 'Latest Alarms')

    def test_home_handles_logged_in_user(self):
        self.request.session = {
            'username': settings.TEST_USER,
            'password': settings.TEST_PASSWORD,
            'api_url': settings.DEFAULT_API_URL}
        response = home(self.request)
        self.request.session = {}
        self.assertContains(response, 'Latest Alarms')
        self.assertNotContains(response, 'id="id_action_login" name="action_login"')

    def test_home_view_login_redirects_to_me_html(self):
        response = home(self.request_post)
        self.assertEqual(response.status_code, 302)
        self.assertTrue(isinstance(response, HttpResponseRedirect))
        self.assertEqual(response.url, '/me/')


class MeViewTests(TestCase):

    @classmethod
    def setUpClass(cls):
        request_factory = RequestFactory()
        cls.request_get = request_factory.get('/me/', follow=True)
        cls.request_post = request_factory.post('/me/', follow=True)
        cls.request_get.session = {}
        cls.request_post.session = {}
        cls.test_api = UwsgiItApi(
            url=settings.DEFAULT_API_URL,
            name='TEST API')
        cls.test_api.save()

    @classmethod
    def tearDownClass(cls):
        cls.test_api.delete()

    def test_me_name_resolves_to_me_url(self):
        url = reverse('console_me')
        self.assertEqual(url, '/me/')

    def test_me_url_resolves_to_me_view(self):
        resolver = resolve('/me/')
        self.assertEqual(resolver.func, me_page)

    def test_me_doesnt_allow_anonymous(self):
        response = me_page(self.request_get)
        self.assertEqual(response.status_code, 302)
        self.assertTrue(isinstance(response, HttpResponseRedirect))
        self.assertEqual(response.url, '/')

        response = me_page(self.request_post)
        self.assertEqual(response.status_code, 302)
        self.assertTrue(isinstance(response, HttpResponseRedirect))
        self.assertEqual(response.url, '/')

    def test_me_handles_logged_in_user(self):
        self.request_get.session = {
            'username': settings.TEST_USER,
            'password': settings.TEST_PASSWORD,
            'api_url': settings.DEFAULT_API_URL}
        response = me_page(self.request_get)
        self.request_get.session = {}
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, '<tr><th><label for="id_company">Company:')


class DomainsViewTests(TestCase):

    @classmethod
    def setUpClass(cls):
        request_factory = RequestFactory()
        cls.request_get = request_factory.get('/domains/', follow=True)
        cls.request_post = request_factory.post('/domains/', follow=True)
        cls.request_get.session = {}
        cls.request_post.session = {}
        cls.test_api = UwsgiItApi(
            url=settings.DEFAULT_API_URL,
            name='TEST API')
        cls.test_api.save()

    @classmethod
    def tearDownClass(cls):
        cls.test_api.delete()

    def test_me_name_resolves_to_me_url(self):
        url = reverse('console_domains')
        self.assertEqual(url, '/domains/')

    def test_domains_url_resolves_to_domains_view(self):
        resolver = resolve('/domains/')
        self.assertEqual(resolver.func, domains)

    def test_domains_doesnt_allow_anonymous(self):
        response = domains(self.request_get)
        self.assertEqual(response.status_code, 302)
        self.assertTrue(isinstance(response, HttpResponseRedirect))
        self.assertEqual(response.url, '/')

        response = domains(self.request_post)
        self.assertEqual(response.status_code, 302)
        self.assertTrue(isinstance(response, HttpResponseRedirect))
        self.assertEqual(response.url, '/')

    def test_domains_handles_logged_in_user(self):
        self.request_get.session = {
            'username': settings.TEST_USER,
            'password': settings.TEST_PASSWORD,
            'api_url': settings.DEFAULT_API_URL}
        response = domains(self.request_get)
        self.request_get.session = {}
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, NewDomainForm())


class TagsViewTests(TestCase):

    @classmethod
    def setUpClass(cls):
        request_factory = RequestFactory()
        cls.request_get = request_factory.get('/tags/', follow=True)
        cls.request_post = request_factory.post('/tags/', follow=True)
        cls.request_get.session = {}
        cls.request_post.session = {}
        cls.test_api = UwsgiItApi(
            url=settings.DEFAULT_API_URL,
            name='TEST API')
        cls.test_api.save()

    @classmethod
    def tearDownClass(cls):
        cls.test_api.delete()

    def test_me_name_resolves_to_me_url(self):
        url = reverse('console_tags')
        self.assertEqual(url, '/tags/')

    def test_tags_url_resolves_to_tags_view(self):
        resolver = resolve('/tags/')
        self.assertEqual(resolver.func, tags)

    def test_tags_doesnt_allow_anonymous(self):
        response = tags(self.request_get)
        self.assertEqual(response.status_code, 302)
        self.assertTrue(isinstance(response, HttpResponseRedirect))
        self.assertEqual(response.url, '/')

        response = tags(self.request_post)
        self.assertEqual(response.status_code, 302)
        self.assertTrue(isinstance(response, HttpResponseRedirect))
        self.assertEqual(response.url, '/')

    def test_tags_handles_logged_in_user(self):
        self.request_get.session = {
            'username': settings.TEST_USER,
            'password': settings.TEST_PASSWORD,
            'api_url': settings.DEFAULT_API_URL}
        response = tags(self.request_get)
        self.request_get.session = {}
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, TagForm())


class Tag_ViewTests(TestCase):

    @classmethod
    def setUpClass(cls):
        request_factory = RequestFactory()
        cls.url = '/tags/{}'.format(settings.TEST_TAG)
        cls.request_get = request_factory.get(cls.url, follow=True)
        cls.request_post = request_factory.post(cls.url, follow=True)
        cls.request_get.session = {}
        cls.request_post.session = {}
        cls.test_api = UwsgiItApi(
            url=settings.DEFAULT_API_URL,
            name='TEST API')
        cls.test_api.save()

    @classmethod
    def tearDownClass(cls):
        cls.test_api.delete()

    def test_tag_name_resolves_to_metrics_url(self):
        url = reverse('console_tag', args=(settings.TEST_TAG,))
        self.assertEqual(url, self.url)

    def test_tag_url_resolves_to_metrics_view(self):
        resolver = resolve(self.url)
        self.assertEqual(resolver.func, tag)

    def test_tag_doesnt_allow_anonymous(self):
        response = tag(self.request_get, settings.TEST_TAG)
        self.assertEqual(response.status_code, 302)
        self.assertTrue(isinstance(response, HttpResponseRedirect))
        self.assertEqual(response.url, '/')

        response = tag(self.request_post, settings.TEST_TAG)
        self.assertEqual(response.status_code, 302)
        self.assertTrue(isinstance(response, HttpResponseRedirect))
        self.assertEqual(response.url, '/')

    def test_tag_handles_logged_in_user(self):
        self.request_get.session = {
            'username': settings.TEST_USER,
            'password': settings.TEST_PASSWORD,
            'api_url': settings.DEFAULT_API_URL}
        response = tag(self.request_get, settings.TEST_TAG)
        self.request_get.session = {}
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, '<h3><b>{tag}</b> Tag</h3>'.format(tag=settings.TEST_TAG))


class Domain_ViewTests(TestCase):

    @classmethod
    def setUpClass(cls):
        request_factory = RequestFactory()
        cls.url = '/domains/{}'.format(settings.TEST_DOMAIN)
        cls.request_get = request_factory.get(cls.url, follow=True)
        cls.request_post = request_factory.post(cls.url, follow=True)
        cls.request_get.session = {}
        cls.request_post.session = {}
        cls.test_api = UwsgiItApi(
            url=settings.DEFAULT_API_URL,
            name='TEST API')
        cls.test_api.save()

    @classmethod
    def tearDownClass(cls):
        cls.test_api.delete()

    def test_domain_name_resolves_to_metrics_url(self):
        url = reverse('console_domain', args=(settings.TEST_DOMAIN,))
        self.assertEqual(url, self.url)

    def test_domain_url_resolves_to_metrics_view(self):
        resolver = resolve(self.url)
        self.assertEqual(resolver.func, domain)

    def test_domain_doesnt_allow_anonymous(self):
        response = domain(self.request_get, settings.TEST_DOMAIN)
        self.assertEqual(response.status_code, 302)
        self.assertTrue(isinstance(response, HttpResponseRedirect))
        self.assertEqual(response.url, '/')

        response = domain(self.request_post, settings.TEST_DOMAIN)
        self.assertEqual(response.status_code, 302)
        self.assertTrue(isinstance(response, HttpResponseRedirect))
        self.assertEqual(response.url, '/')

    def test_domain_handles_logged_in_user(self):
        self.request_get.session = {
            'username': settings.TEST_USER,
            'password': settings.TEST_PASSWORD,
            'api_url': settings.DEFAULT_API_URL}
        response = domain(self.request_get, settings.TEST_DOMAIN)
        self.request_get.session = {}
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, '<td>{id}</td>'.format(id=settings.TEST_DOMAIN))


class Containers_ViewTests(TestCase):

    @classmethod
    def setUpClass(cls):
        request_factory = RequestFactory()
        cls.url = '/containers/{}'.format(settings.TEST_CONTAINER)
        cls.request_get = request_factory.get(cls.url, follow=True)
        cls.request_post = request_factory.post(cls.url, follow=True)
        cls.request_get.session = {}
        cls.request_post.session = {}
        cls.test_api = UwsgiItApi(
            url=settings.DEFAULT_API_URL,
            name='TEST API')
        cls.test_api.save()

    @classmethod
    def tearDownClass(cls):
        cls.test_api.delete()

    def test_containers_name_resolves_to_containers_url(self):
        url = reverse('console_containers', args=(settings.TEST_CONTAINER,))
        self.assertEqual(url, self.url)

    def test_containers_url_resolves_to_containers_view(self):
        resolver = resolve(self.url)
        self.assertEqual(resolver.func, containers)

    def test_containers_doesnt_allow_anonymous(self):
        response = containers(self.request_get, settings.TEST_CONTAINER)
        self.assertEqual(response.status_code, 302)
        self.assertTrue(isinstance(response, HttpResponseRedirect))
        self.assertEqual(response.url, '/')

        response = containers(self.request_post, settings.TEST_CONTAINER)
        self.assertEqual(response.status_code, 302)
        self.assertTrue(isinstance(response, HttpResponseRedirect))
        self.assertEqual(response.url, '/')

    def test_containers_handles_logged_in_user(self):
        self.request_get.session = {
            'username': settings.TEST_USER,
            'password': settings.TEST_PASSWORD,
            'api_url': settings.DEFAULT_API_URL}
        response = containers(self.request_get, settings.TEST_CONTAINER)
        self.request_get.session = {}
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, '({id})</b>'.format(id=settings.TEST_CONTAINER))


class AlarmsViewTests(TestCase):

    @classmethod
    def setUpClass(cls):
        request_factory = RequestFactory()
        cls.url = '/alarms/'
        cls.request_get = request_factory.get(cls.url, follow=True)
        cls.request_post = request_factory.post(cls.url, follow=True)
        cls.request_get.session = {}
        cls.request_post.session = {}
        cls.test_api = UwsgiItApi(
            url=settings.DEFAULT_API_URL,
            name='TEST API')
        cls.test_api.save()

    @classmethod
    def tearDownClass(cls):
        cls.test_api.delete()

    def test_alarms_name_resolves_to_alarms_url(self):
        url = reverse('console_alarms')
        self.assertEqual(url, self.url)

    def test_alarms_url_resolves_to_alarms_view(self):
        resolver = resolve(self.url)
        self.assertEqual(resolver.func, alarms)

    def test_alarms_doesnt_allow_anonymous(self):
        response = alarms(self.request_get)
        self.assertEqual(response.status_code, 302)
        self.assertTrue(isinstance(response, HttpResponseRedirect))
        self.assertEqual(response.url, '/')

        response = alarms(self.request_post)
        self.assertEqual(response.status_code, 302)
        self.assertTrue(isinstance(response, HttpResponseRedirect))
        self.assertEqual(response.url, '/')

    def test_alarms_handles_logged_in_user(self):
        self.request_get.session = {
            'username': settings.TEST_USER,
            'password': settings.TEST_PASSWORD,
            'api_url': settings.DEFAULT_API_URL}
        response = alarms(self.request_get)
        self.request_get.session = {}
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, '<form class="form-horizontal" role="form" method="POST">')


class Latest_alarmsViewTests(TestCase):
    @classmethod
    def setUpClass(cls):
        request_factory = RequestFactory()
        cls.url = '/latest_alarms/'
        cls.request_get = request_factory.get(cls.url, follow=True)
        cls.request_post = request_factory.post(cls.url, follow=True)
        cls.request_get.session = {}
        cls.request_post.session = {}
        cls.test_api = UwsgiItApi(
            url=settings.DEFAULT_API_URL,
            name='TEST API')
        cls.test_api.save()

    @classmethod
    def tearDownClass(cls):
        cls.test_api.delete()

    def test_latest_alarms_name_resolves_to_latest_alarms_url(self):
        url = reverse('console_latest_alarms')
        self.assertEqual(url, self.url)

    def test_latest_alarms_url_resolves_to_latest_alarms_view(self):
        resolver = resolve(self.url)
        self.assertEqual(resolver.func, latest_alarms)

    def test_latest_alarms_doesnt_allow_anonymous(self):
        response = latest_alarms(self.request_get)
        self.assertEqual(response.status_code, 302)
        self.assertTrue(isinstance(response, HttpResponseRedirect))
        self.assertEqual(response.url, '/')

        response = latest_alarms(self.request_post)
        self.assertEqual(response.status_code, 302)
        self.assertTrue(isinstance(response, HttpResponseRedirect))
        self.assertEqual(response.url, '/')

    def test_latest_alarms_handles_logged_in_user(self):
        self.request_get.session = {
            'username': settings.TEST_USER,
            'password': settings.TEST_PASSWORD,
            'api_url': settings.DEFAULT_API_URL}
        response = latest_alarms(self.request_get)
        self.request_get.session = {}
        self.assertEqual(response.status_code, 200)


class ContainerMetricViewTests(TestCase):

    @classmethod
    def setUpClass(cls):
        request_factory = RequestFactory()
        cls.request_get = request_factory.get('/metrics/container/io.read/id/1/', follow=True)
        cls.request_post = request_factory.post(
            '/metrics/container/io.read/id/1/', follow=True,
            data={'year': 2000, 'month': 1, 'day': 1})
        cls.request_get.session = {}
        cls.request_post.session = {}
        json = [
            [1407103492, 12288], [1407103808, 12288], [1407104120, 12288],
            [1407104436, 12288], [1407104748, 12288], [1407105061, 12288],
            [1407105374, 12288], [1407105687, 12288], [1407105997, 12288],
            [1407106309, 12288], [1407106619, 12288], [1407106930, 12288],
            [1407107241, 12288], [1407107552, 12288], [1407107865, 12288],
            [1407108179, 12288], [1407108490, 12288], [1407108805, 12288],
            [1407109116, 12288], [1407109428, 12288], [1407109740, 12288],
            [1407110052, 12288], [1407110366, 12288], [1407110683, 12288],
            [1407110995, 12288], [1407111311, 12288], [1407111621, 12288],
            [1407111932, 12288], [1407112245, 12288], [1407112559, 12288],
            [1407112872, 12288], [1407113183, 12288], [1407113495, 12288],
            [1407113808, 12288], [1407114119, 12288], [1407114431, 12288],
            [1407114744, 12288], [1407115055, 12288], [1407115366, 12288],
            [1407115679, 12288], [1407115993, 12288], [1407116306, 12288],
            [1407116619, 12288], [1407116932, 12288], [1407117246, 12288],
            [1407117560, 12288], [1407117869, 12288], [1407118179, 12288],
            [1407118499, 12288], [1407118819, 12288], [1407119132, 12288],
            [1407119446, 12288], [1407119765, 12288], [1407120082, 12288],
            [1407120398, 12288], [1407120711, 12288], [1407121025, 12288],
            [1407121336, 12288], [1407121651, 12288], [1407121963, 12288],
            [1407122274, 12288], [1407122585, 12288], [1407122897, 12288],
            [1407123209, 12288], [1407123526, 12288], [1407123840, 12288],
            [1407124151, 12288], [1407124464, 12288], [1407124777, 12288],
        ]
        cls.test_metric = IOReadContainerMetric(
            container=1,
            year=2000,
            month=1,
            day=1,
            json=json
        )

        cls.test_metric.save()
        cls.test_api = UwsgiItApi(
            url=settings.DEFAULT_API_URL,
            name='TEST API')
        cls.test_api.save()

    @classmethod
    def tearDownClass(cls):
        cls.test_api.delete()
        cls.test_metric.delete()

    def test_IOReadContainerMetric_name_resolves_to_metrics_url(self):
        url = reverse('console_container_io_read', args=(1,))
        self.assertEqual(url, '/metrics/container/io.read/id/1/')

    def test_IOReadContainerMetric_url_resolves_to_metrics_view(self):
        resolver = resolve('/metrics/container/io.read/id/1/')
        self.assertEqual(resolver.func, container_metrics)

    def test_IOWriteContainerMetric_name_resolves_to_metrics_url(self):
        url = reverse('console_container_io_write', args=(1,))
        self.assertEqual(url, '/metrics/container/io.write/id/1/')

    def test_IOWriteContainerMetric_url_resolves_to_metrics_view(self):
        resolver = resolve('/metrics/container/io.write/id/1/')
        self.assertEqual(resolver.func, container_metrics)

    def test_NetworkRXContainerMetric_name_resolves_to_metrics_url(self):
        url = reverse('console_container_net_rx', args=(1,))
        self.assertEqual(url, '/metrics/container/net.rx/id/1/')

    def test_NetworkRXContainerMetric_url_resolves_to_metrics_view(self):
        resolver = resolve('/metrics/container/net.rx/id/1/')
        self.assertEqual(resolver.func, container_metrics)

    def test_NetworkTXContainerMetric_name_resolves_to_metrics_url(self):
        url = reverse('console_container_net_tx', args=(1,))
        self.assertEqual(url, '/metrics/container/net.tx/id/1/')

    def test_NetworkTXContainerMetric_url_resolves_to_metrics_view(self):
        resolver = resolve('/metrics/container/net.tx/id/1/')
        self.assertEqual(resolver.func, container_metrics)

    def test_CPUContainerMetric_name_resolves_to_metrics_url(self):
        url = reverse('console_container_cpu', args=(1,))
        self.assertEqual(url, '/metrics/container/cpu/id/1/')

    def test_CPUContainerMetric_url_resolves_to_metrics_view(self):
        resolver = resolve('/metrics/container/cpu/id/1/')
        self.assertEqual(resolver.func, container_metrics)

    def test_MemoryContainerMetric_name_resolves_to_metrics_url(self):
        url = reverse('console_container_mem', args=(1,))
        self.assertEqual(url, '/metrics/container/mem/id/1/')

    def test_MemoryContainerMetric_url_resolves_to_metrics_view(self):
        resolver = resolve('/metrics/container/mem/id/1/')
        self.assertEqual(resolver.func, container_metrics)

    def test_IOReadContainerMetricPerTag_name_resolves_to_metrics_url(self):
        url = reverse('console_container_io_read_per_tag', args=(settings.TEST_TAG,))
        self.assertEqual(url, '/metrics/container/io.read/tag/{}/'.format(settings.TEST_TAG))

    def test_IOReadContainerMetricPerTag_url_resolves_to_metrics_view(self):
        resolver = resolve('/metrics/container/io.read/tag/{}/'.format(settings.TEST_TAG))
        self.assertEqual(resolver.func, container_metrics_per_tag)

    def test_IOWriteContainerMetricPerTag_name_resolves_to_metrics_url(self):
        url = reverse('console_container_io_write_per_tag', args=(settings.TEST_TAG,))
        self.assertEqual(url, '/metrics/container/io.write/tag/{}/'.format(settings.TEST_TAG))

    def test_IOWriteContainerMetricPerTag_url_resolves_to_metrics_view(self):
        resolver = resolve('/metrics/container/io.write/tag/{}/'.format(settings.TEST_TAG))
        self.assertEqual(resolver.func, container_metrics_per_tag)

    def test_NetworkRXContainerMetricPerTag_name_resolves_to_metrics_url(self):
        url = reverse('console_container_net_rx_per_tag', args=(settings.TEST_TAG,))
        self.assertEqual(url, '/metrics/container/net.rx/tag/{}/'.format(settings.TEST_TAG))

    def test_NetworkRXContainerMetricPerTag_url_resolves_to_metrics_view(self):
        resolver = resolve('/metrics/container/net.rx/tag/{}/'.format(settings.TEST_TAG))
        self.assertEqual(resolver.func, container_metrics_per_tag)

    def test_NetworkTXContainerMetricPerTag_name_resolves_to_metrics_url(self):
        url = reverse('console_container_net_tx_per_tag', args=(settings.TEST_TAG,))
        self.assertEqual(url, '/metrics/container/net.tx/tag/{}/'.format(settings.TEST_TAG))

    def test_NetworkTXContainerMetricPerTag_url_resolves_to_metrics_view(self):
        resolver = resolve('/metrics/container/net.tx/tag/{}/'.format(settings.TEST_TAG))
        self.assertEqual(resolver.func, container_metrics_per_tag)

    def test_CPUContainerMetricPerTag_name_resolves_to_metrics_url(self):
        url = reverse('console_container_cpu_per_tag', args=(settings.TEST_TAG,))
        self.assertEqual(url, '/metrics/container/cpu/tag/{}/'.format(settings.TEST_TAG))

    def test_CPUContainerMetricPerTag_url_resolves_to_metrics_view(self):
        resolver = resolve('/metrics/container/cpu/tag/{}/'.format(settings.TEST_TAG))
        self.assertEqual(resolver.func, container_metrics_per_tag)

    def test_MemoryContainerMetricPerTag_name_resolves_to_metrics_url(self):
        url = reverse('console_container_mem_per_tag', args=(settings.TEST_TAG,))
        self.assertEqual(url, '/metrics/container/mem/tag/{}/'.format(settings.TEST_TAG))

    def test_MemoryContainerMetricPerTag_url_resolves_to_metrics_view(self):
        resolver = resolve('/metrics/container/mem/tag/{}/'.format(settings.TEST_TAG))
        self.assertEqual(resolver.func, container_metrics_per_tag)

    def test_container_view_doesnt_allows_anonymous(self):
        response = container_metrics(
            self.request_get, 1, **{'model': IOReadContainerMetric, 'absolute_values': False, 'average': False})
        self.assertEqual(response.status_code, 302)
        self.assertTrue(isinstance(response, HttpResponseRedirect))
        self.assertEqual(response.url, '/')

        response = container_metrics(
            self.request_post, 1, **{'model': IOReadContainerMetric, 'absolute_values': False, 'average': False})
        self.assertEqual(response.status_code, 302)
        self.assertTrue(isinstance(response, HttpResponseRedirect))
        self.assertEqual(response.url, '/')

    def test_container_view_handles_logged_in_user(self):
        self.request_post.session = {
            'username': settings.TEST_USER,
            'password': settings.TEST_PASSWORD,
            'api_url': self.test_api.id}
        response = container_metrics(
            self.request_post, 1, **{'model': IOReadContainerMetric, 'absolute_values': False, 'average': False})
        self.request_post.session = {}
        self.assertEqual(response.status_code, 200)


class DomainMetricViewTests(TestCase):

    @classmethod
    def setUpClass(cls):
        request_factory = RequestFactory()
        cls.request_get = request_factory.get('/metrics/domain/net.rx/id/1/', follow=True)
        cls.request_post = request_factory.post(
            '/metrics/domain/net.rx/id/1/', follow=True,
            data={'year': 2000, 'month': 1, 'day': 1})
        cls.request_get.session = {}
        cls.request_post.session = {}
        json = [
            [1407103492, 12288], [1407103808, 12288], [1407104120, 12288],
            [1407104436, 12288], [1407104748, 12288], [1407105061, 12288],
            [1407105374, 12288], [1407105687, 12288], [1407105997, 12288],
            [1407106309, 12288], [1407106619, 12288], [1407106930, 12288],
            [1407107241, 12288], [1407107552, 12288], [1407107865, 12288],
            [1407108179, 12288], [1407108490, 12288], [1407108805, 12288],
            [1407109116, 12288], [1407109428, 12288], [1407109740, 12288],
            [1407110052, 12288], [1407110366, 12288], [1407110683, 12288],
            [1407110995, 12288], [1407111311, 12288], [1407111621, 12288],
            [1407111932, 12288], [1407112245, 12288], [1407112559, 12288],
            [1407112872, 12288], [1407113183, 12288], [1407113495, 12288],
            [1407113808, 12288], [1407114119, 12288], [1407114431, 12288],
            [1407114744, 12288], [1407115055, 12288], [1407115366, 12288],
            [1407115679, 12288], [1407115993, 12288], [1407116306, 12288],
            [1407116619, 12288], [1407116932, 12288], [1407117246, 12288],
            [1407117560, 12288], [1407117869, 12288], [1407118179, 12288],
            [1407118499, 12288], [1407118819, 12288], [1407119132, 12288],
            [1407119446, 12288], [1407119765, 12288], [1407120082, 12288],
            [1407120398, 12288], [1407120711, 12288], [1407121025, 12288],
            [1407121336, 12288], [1407121651, 12288], [1407121963, 12288],
            [1407122274, 12288], [1407122585, 12288], [1407122897, 12288],
            [1407123209, 12288], [1407123526, 12288], [1407123840, 12288],
            [1407124151, 12288], [1407124464, 12288], [1407124777, 12288],
        ]

        cls.test_metric = NetworkRXDomainMetric(
            domain=1,
            container=1,
            year=2000,
            month=1,
            day=1,
            json=json
        )

        cls.test_metric.save()
        cls.test_api = UwsgiItApi(
            url=settings.DEFAULT_API_URL,
            name='TEST API')
        cls.test_api.save()

    @classmethod
    def tearDownClass(cls):
        cls.test_api.delete()
        cls.test_metric.delete()

    def test_NetworkRXDomainMetric_name_resolves_to_metrics_url(self):
        url = reverse('console_domain_net_rx', args=(1,))
        self.assertEqual(url, '/metrics/domain/net.rx/id/1/')

    def test_NetworkRXDomainMetric_url_resolves_to_metrics_view(self):
        resolver = resolve('/metrics/domain/net.rx/id/1/')
        self.assertEqual(resolver.func, domain_metrics)

    def test_NetworkTXDomainMetric_name_resolves_to_metrics_url(self):
        url = reverse('console_domain_net_tx', args=(1,))
        self.assertEqual(url, '/metrics/domain/net.tx/id/1/')

    def test_NetworkTXDomainMetric_url_resolves_to_metrics_view(self):
        resolver = resolve('/metrics/domain/net.tx/id/1/')
        self.assertEqual(resolver.func, domain_metrics)

    def test_HitsDomainMetric_name_resolves_to_metrics_url(self):
        url = reverse('console_domain_hits', args=(1,))
        self.assertEqual(url, '/metrics/domain/hits/id/1/')

    def test_HitsDomainMetric_url_resolves_to_metrics_view(self):
        resolver = resolve('/metrics/domain/hits/id/1/')
        self.assertEqual(resolver.func, domain_metrics)

    def test_NetworkRXDomainMetricPerTag_name_resolves_to_metrics_url(self):
        url = reverse('console_domain_net_rx_per_tag', args=(settings.TEST_TAG,))
        self.assertEqual(url, '/metrics/domain/net.rx/tag/{}/'.format(settings.TEST_TAG))

    def test_NetworkRXDomainMetricPerTag_url_resolves_to_metrics_view(self):
        resolver = resolve('/metrics/domain/net.rx/tag/{}/'.format(settings.TEST_TAG))
        self.assertEqual(resolver.func, domain_metrics_per_tag)

    def test_NetworkTXDomainMetricPerTag_name_resolves_to_metrics_url(self):
        url = reverse('console_domain_net_tx_per_tag', args=(settings.TEST_TAG,))
        self.assertEqual(url, '/metrics/domain/net.tx/tag/{}/'.format(settings.TEST_TAG))

    def test_NetworkTXDomainMetricPerTag_url_resolves_to_metrics_view(self):
        resolver = resolve('/metrics/domain/net.tx/tag/{}/'.format(settings.TEST_TAG))
        self.assertEqual(resolver.func, domain_metrics_per_tag)

    def test_HitsDomainMetricPerTag_name_resolves_to_metrics_url(self):
        url = reverse('console_domain_hits_per_tag', args=(settings.TEST_TAG,))
        self.assertEqual(url, '/metrics/domain/hits/tag/{}/'.format(settings.TEST_TAG))

    def test_HitsDomainMetricPerTag_url_resolves_to_metrics_view(self):
        resolver = resolve('/metrics/domain/hits/tag/{}/'.format(settings.TEST_TAG))
        self.assertEqual(resolver.func, domain_metrics_per_tag)

    def test_domain_view_doesnt_allows_anonymous(self):
        response = domain_metrics(
            self.request_get, 1, **{'model': NetworkRXDomainMetric, 'absolute_values': False, 'average': False})
        self.assertEqual(response.status_code, 302)
        self.assertTrue(isinstance(response, HttpResponseRedirect))
        self.assertEqual(response.url, '/')

        response = domain_metrics(
            self.request_post, 1, **{'model': NetworkRXDomainMetric, 'absolute_values': False, 'average': False})
        self.assertEqual(response.status_code, 302)
        self.assertTrue(isinstance(response, HttpResponseRedirect))
        self.assertEqual(response.url, '/')

    def test_domain_view_handles_logged_in_user(self):
        self.request_post.session = {
            'username': settings.TEST_USER,
            'password': settings.TEST_PASSWORD,
            'api_url': self.test_api.id}
        response = domain_metrics(
            self.request_post, 1, **{'model': NetworkRXDomainMetric, 'absolute_values': False, 'average': False})
        self.request_post.session = {}
        self.assertEqual(response.status_code, 200)


class LogoutViewTest(TestCase):

    @classmethod
    def setUpClass(cls):
        cls.test_api = UwsgiItApi(
            url=settings.DEFAULT_API_URL,
            name='TEST API')
        cls.test_api.save()

        request_factory = RequestFactory()
        cls.request_get = request_factory.get('/logout/', follow=True)
        cls.request_get.session = {}
        cls.request_post = request_factory.post('/logout/', follow=True)
        cls.request_post.session = {}

    @classmethod
    def tearDownClass(cls):
        cls.test_api.delete()

    def test_logout_name_resolves_to_logout_url(self):
        url = reverse('console_logout')
        self.assertEqual(url, '/logout/')

    def test_logout_url_resolves_to_logout_view(self):
        resolver = resolve('/logout/')
        self.assertEqual(resolver.func, logout)

    def test_logout_redirects_anonymous(self):
        response = tags(self.request_post)
        self.assertEqual(response.status_code, 302)
        self.assertTrue(isinstance(response, HttpResponseRedirect))
        self.assertEqual(response.url, '/')

        response = tags(self.request_post)
        self.assertEqual(response.status_code, 302)
        self.assertTrue(isinstance(response, HttpResponseRedirect))
        self.assertEqual(response.url, '/')

    def test_logout_logs_out_logged_in_user(self):
        #login
        self.client.post('/', follow=True, data={
            'username': settings.TEST_USER,
            'password': settings.TEST_PASSWORD,
            'api_url': self.test_api.id,
            'action_login': 1})
        response = self.client.get('/me/')
        self.assertEqual(response.status_code, 200)

        response = self.client.get('/logout/', follow=True)
        self.assertRedirects(response, '/')
        response = self.client.get('/me/', follow=True)
        self.assertRedirects(response, '/')

        #login
        self.client.post('/', follow=True, data={
            'username': settings.TEST_USER,
            'password': settings.TEST_PASSWORD,
            'api_url': self.test_api.id,
            'action_login': 1})
        response = self.client.get('/me/')
        self.assertEqual(response.status_code, 200)

        response = self.client.post('/logout/', follow=True)
        self.assertRedirects(response, '/')
        response = self.client.get('/me/', follow=True)
        self.assertRedirects(response, '/')
